--                     ͻ
--                       CORPS DU PAQUETAGE ALGEBLIN  
--                     ͼ

with TEXT_IO, IIO, FIO, MATH1, COMMON_MATH0, MATH0, MATRIX;
use TEXT_IO, COMMON_MATH0, MATH0, MATRIX;

package body ALGEBLIN is

procedure SYSTEME_TRIANGULAIRE_SUPERIEUR(A: in MATRICE; b: in VECTEUR;
					 x: out VECTEUR) is
S: FLOAT;
begin
    if (A'LENGTH(1) /= A'LENGTH(2)) or else (A'LENGTH(1) /= b'LENGTH) then
       PUT_LINE("Erreur dans SYSTEME_TRIANGULAIRE_SUPERIEUR: Dimensions incorrectes");
       raise ERREUR_DIMENSIONS;
    end if;
    begin
       x(x'LAST) := b(x'LAST)/A(x'LAST,x'LAST);
    exception
       when NUMERIC_ERROR =>
	 PUT_LINE("Erreur dans SYSTEME_TRIANGULAIRE_SUPERIEUR: Determinant = 0");
	 raise; -- On propage l'erreur
    end;
    for i in reverse x'FIRST..x'LAST - 1
    loop
       S := 0.0;
       for j in i + 1..x'LAST
       loop
	  S := S + A(i,j)*b(j);
       end loop;
       begin
	  x(i) := (b(i) - S)/A(i,i);
       exception
	  when NUMERIC_ERROR =>
	  PUT_LINE("Erreur dans SYSTEME_TRIANGULAIRE_SUPERIEUR: Determinant = 0");
	  raise; -- On propage l'erreur
       end;
    end loop;
end SYSTEME_TRIANGULAIRE_SUPERIEUR;
-- 
procedure PIVOT_MAX(k: in NATURAL; A: in out MATRICE; b: in out VECTEUR) is
EPS: constant FLOAT := 1.0E-12;
L: NATURAL := k;
begin
    for i in k + 1..A'LAST(1)
    loop
       if ( abs(A(i,k)) > abs(A(L,k)) ) then
	  L := i;
       end if;
    end loop;
    if (abs(A(L,k)) < EPS) then
       PUT_LINE("Erreur dans PIVOT_MAX: Pivot trop petit");
       raise ERREUR_DETERMINANT_NUL;
    end if;
    if (L /= k) then
       for j in k..A'LAST(2)
       loop
	  ECHANGER_REELS(A(k,j),A(L,j));
       end loop;
       ECHANGER_REELS(b(k),b(L));
    end if;
end PIVOT_MAX;
-- 
procedure ELIMINER(k: in NATURAL; A: in out MATRICE; b: in out VECTEUR) is
M: FLOAT;
begin
    for i in k + 1..A'LAST(2)
    loop
       M := A(i,k)/A(k,k);
       for j in k + 1..A'LAST(2)
       loop
	  A(i,j) := A(i,j) - M*A(k,j);
       end loop;
       A(i,k) := 0.0;
       b(i) := b(i) - M*b(k);
    end loop;
end ELIMINER;
-- 
procedure PIVOTAGE(A: in out MATRICE; b: in out VECTEUR) is
begin
    for k in b'FIRST..b'LAST - 1
    loop
	PIVOT_MAX(k,A,b);
	ELIMINER(k,A,b);
    end loop;
end PIVOTAGE;
-- 
function DET_MATRICE_TRIANGULAIRE(A: in MATRICE) return FLOAT is
Aux: FLOAT;
begin
   Aux := A(1,1);
   for i in A'FIRST(1) + 1..A'LAST(1)
   loop
      Aux := Aux*A(i,i);
   end loop;
   return Aux;
end DET_MATRICE_TRIANGULAIRE;
-- 
function INV_MATRICE_TRIANG_INF(A: in MATRICE) return MATRICE is
EPS: constant FLOAT := 1.0E-12;
Inv: MATRICE(A'range(1),A'range(2));
S: FLOAT;
begin
   for i in A'range(1)
   loop
     begin
	Inv(i,i) := 1.0/A(i,i);
     exception
	when NUMERIC_ERROR =>
	  PUT_LINE("Erreur dans INVERSE_MATRICE_TRIANGULAIRE: Determinant = 0");
	  raise;
     end;
     for j in i + 1..A'LAST(2)
     loop
	Inv(i,j) := 0.0;
     end loop;
   end loop;
   for i in A'FIRST(1)..A'LAST(1) - 1
   loop
      for k in i + 1..A'LAST(1)
      loop
	S := 0.0;
	for j in i..k - 1
	loop
	   S := S + A(k,j)*Inv(j,i);
	end loop;
	Inv(k,i) := -S/A(k,k);
      end loop;
   end loop;
   return Inv;
end INV_MATRICE_TRIANG_INF;
-- 
function TRANSPOSE(A: in MATRICE) return MATRICE is
TA: MATRICE(A'range(1),A'range(2));
begin
    for i in A'range(1)
    loop
       for j in A'range(2)
       loop
	  TA(i,j) := A(j,i);
       end loop;
     end loop;
     return TA;
end TRANSPOSE;
-- 
function INV_MATRICE_TRIANG_SUP(A: in MATRICE) return MATRICE is
begin
    return TRANSPOSE(INV_MATRICE_TRIANG_INF(TRANSPOSE(A)));
end INV_MATRICE_TRIANG_SUP;
-- 
procedure DECOMPOSITION_LR(A: in MATRICE; L, R: in out MATRICE) is
EPS: constant FLOAT := 1.0E-12;
S: FLOAT;
begin
    for i in A'range(1)
    loop
       L(i,i) := 1.0;
       for j in i + 1..A'LAST(2)
       loop
	  L(i,j) := 0.0;
       end loop;
       for j in A'FIRST(2)..i - 1
       loop
	  R(i,j) := 0.0;
       end loop;
    end loop;
    for j in A'range(2)
    loop
       R(1,j) := A(1,j);
    end loop;
    for i in A'FIRST(1) + 1..A'LAST(1)
    loop
       L(i,1) := A(i,1)/A(1,1);
    end loop;
    for k in A'FIRST(1) + 1..A'LAST(1)
    loop
       for j in k..A'LAST(2)
       loop
	  S := 0.0;
	  for i in A'FIRST(1)..k - 1
	  loop
	     S := S + R(i,j)*L(k,i);
	  end loop;
	  R(k,j) := A(k,j) - S;
       end loop;
       if (k < A'LAST(1)) then
	  for i in k + 1..A'LAST(1)
	  loop
	     S := 0.0;
	     for j in A'FIRST(1)..k - 1
	     loop
		S := S + L(i,j)*R(j,k);
	     end loop;
	     if (abs(R(k,k)) < EPS) then
		PUT_LINE("Erreur dans DECOMPOSITION_LR: Decomposition impossible");
		raise ERREUR_DETERMINANT_NUL;
	     end if;
	     L(i,k) := (A(i,k) - S)/R(k,k);
	  end loop;
       end if;
    end loop;
end DECOMPOSITION_LR;
-- 
function DETERMINANT_LR(A: in MATRICE) return FLOAT is
begin
   declare L, R: MATRICE(A'range(1),A'range(2));
   begin
      DECOMPOSITION_LR(A,L,R);
      return DET_MATRICE_TRIANGULAIRE(R);
   end ;
end DETERMINANT_LR;
-- 
function INVERSE_LR(A: in MATRICE) return MATRICE is
begin
   declare L, R: MATRICE(A'range(1),A'range(2));
   begin
      DECOMPOSITION_LR(A,L,R);
      return INV_MATRICE_TRIANG_SUP(R)*INV_MATRICE_TRIANG_INF(L);
    end;
end INVERSE_LR;
-- 
procedure GAUSS_JORDAN(A: in out MATRICE; b: in out VECTEUR) is

procedure ELIMINER_GJ(k: in NATURAL; A: in out MATRICE; b: in out VECTEUR) is
begin
    for j in k + 1..A'LAST(2)
    loop
       A(k,j) := A(k,j)/A(k,k);
    end loop;
    b(k) := b(k)/A(k,k);
    A(k,k) := 1.0;
    for i in A'FIRST(1)..k - 1
    loop
       for j in k + 1..A'LAST(2)
       loop
	  A(i,j) := A(i,j) - A(i,k)*A(k,j);
       end loop;
       b(i) := b(i) - A(i,k)*b(k);
       A(i,k) := 0.0;
    end loop;
    for i in k + 1..A'LAST(1)
    loop
       for j in k + 1..A'LAST(2)
       loop
	  A(i,j) := A(i,j) - A(i,k)*A(k,j);
       end loop;
       b(i) := b(i) - A(i,k)*b(k);
       A(i,k) := 0.0;
    end loop;
end ELIMINER_GJ;

begin
    for k in A'range(1)
    loop
       PIVOT_MAX(k,A,b);
       ELIMINER_GJ(k,A,b);
    end loop;
end GAUSS_JORDAN;
-- 
function INVERSE_GAUSS_JORDAN(A: in MATRICE) return MATRICE is
k: NATURAL;
AA, INV: MATRICE(A'range(1),A'range(2));

procedure PIVOT_MAX_GJ(k: in NATURAL; A, INV: in out MATRICE) is
L: NATURAL := k;
EPS: constant FLOAT := 1.0E-12;
begin
    for i in k + 1..A'LAST(1)
    loop
       if (abs(A(i,k)) > abs(A(L,k))) then
	 L := i;
       end if;
    end loop;
    if (abs(A(L,k)) < EPS) then
       PUT_LINE("Erreur dans INVERSE_GAUSS_JORDAN: Pivot trop petit");
       raise ERREUR_DETERMINANT_NUL;
    end if;
    if (L /= k) then
       for j in k..A'LAST(2)
       loop
	  ECHANGER_REELS(A(k,j),A(L,j));
       end loop;
       for j in A'range(2)
       loop
	  ECHANGER_REELS(INV(k,j),INV(L,j));
       end loop;
     end if;
end PIVOT_MAX_GJ;

-- Elimination des coefficients de x(k) dans les lignes 1 a k - 1 et k + 1
-- a n.

procedure ELIMINER_INV_GJ(k: in NATURAL; A, INV: in out MATRICE) is
begin
    for j in k + 1..A'LAST(2)
    loop
       A(k,j) := A(k,j)/A(k,k);
    end loop;
    for j in A'range(1)
    loop
       INV(k,j) := INV(k,j)/A(k,k);
    end loop;
    A(k,k) := 1.0;
    for i in A'FIRST(1)..k - 1
    loop
       for j in k + 1..A'LAST(2)
       loop
	  A(i,j) := A(i,j) - A(i,k)*A(k,j);
       end loop;
       for j in A'range(2)
       loop
	  INV(i,j) := INV(i,j) - A(i,k)*INV(k,j);
       end loop;
       A(i,k) := 0.0;
     end loop;
     for i in k + 1..A'LAST(1)
     loop
	for j in k + 1..A'LAST(2)
	loop
	  A(i,j) := A(i,j) - A(i,k)*A(k,j);
	end loop;
	for j in A'range(2)
	loop
	  INV(i,j) := INV(i,j) - A(i,k)*INV(k,j);
	end loop;
	A(i,k) := 0.0;
     end loop;
end ELIMINER_INV_GJ;

begin
    AA := A;
    INV := (others => (others => 0.0));
    for i in A'range(1)
    loop
       INV(i,i) := 1.0;
    end loop;
    for k in A'range(1)
    loop
       PIVOT_MAX_GJ(k,AA,INV);
       ELIMINER_INV_GJ(k,AA,INV);
    end loop;
    return INV;
end INVERSE_GAUSS_JORDAN;
-- 
procedure SYSTEME_TRIDIAGONAL(A: in out MATRICE_TRIDIAGONALE;
			      b: in out VECTEUR) is
EPS: constant FLOAT := 1.0E-12;
m: FLOAT;
begin
    for i in A'FIRST + 1..A'LAST
    loop
       if (abs(A(i - 1)(2)) < EPS) then
	  PUT_LINE("Erreur dans SYSTEME_TRIDIAGONAL: Pivot trop petit");
	  raise ERREUR_DETERMINANT_NUL;
       end if;
       m := A(i)(1)/A(i - 1)(2);
       A(i)(2) := A(i)(2) - m*A(i - 1)(3);
       b(i) := b(i) - m*b(i - 1);
    end loop;
    if (abs(A(A'LAST)(2)) < EPS) then
      PUT_LINE("Erreur dans SYSTEME_TRIDIAGONAL: Determinant nul");
      raise ERREUR_DETERMINANT_NUL;
    end if;
    b(A'LAST) := b(A'LAST)/A(A'LAST)(2);
    for i in reverse A'FIRST..A'LAST - 1
    loop
       if (abs(A(i)(2)) < EPS) then
	  PUT_LINE("Erreur dans SYSTEME_TRIDIAGONAL: Determinant nul");
	  raise ERREUR_DETERMINANT_NUL;
       end if;
       b(i) := (b(i) - A(i)(3)*b(i + 1))/A(i)(2);
    end loop;
end SYSTEME_TRIDIAGONAL;
-- 
procedure CHOLESKY(A: in MATRICE; C: in out MATRICE) is
S: FLOAT;
begin
   C := (others => (others => 0.0));
   for i in A'range(1)
   loop
      for j in A'FIRST(2)..i - 1
      loop
	 S := A(i,j);
	 for k in A'FIRST(2)..j - 1
	 loop
	    S := S - C(i,k)*C(j,k);
	 end loop;
	 C(i,j) := S/C(j,j);
      end loop;
      S := A(i,i);
      for k in A'FIRST(2)..i - 1
      loop
	 S := S - C(i,k)*C(i,k);
      end loop;
      C(i,i) := MATH1.SQRT(S);
   end loop;
end CHOLESKY;
-- 
function INVERSE_VAN_DER_MONDE(x: in VECTEUR) return MATRICE is
INV: MATRICE(x'range,x'range);
Pol: VECTEUR(x'FIRST..x'LAST + 1) := (others => 0.0);
Di: FLOAT;

procedure MASTER_POL(x: in VECTEUR; Pol: in out VECTEUR) is
begin
    Pol(1) := -x(1);
    Pol(2) := 1.0;
    for k in x'FIRST + 1..x'LAST
    loop
       for j in reverse x'FIRST..k - 1
       loop
	   Pol(j + 1) := Pol(j) - x(k)*Pol(j + 1);
       end loop;
       Pol(k + 1) := 1.0;
       Pol(1) := -Pol(1)*x(k);
    end loop;
end MASTER_POL;

begin
    MASTER_POL(x,Pol);
    for i in x'range
    loop
       Di := 1.0;
       Inv(i,x'LAST) := 1.0;
       for j in x'range
       loop
	  if (j /= i) then
	     Di := Di*(x(i) - x(j));
	  end if;
       end loop;
       if (x(i) = 0.0) then
	 for j in x'FIRST..x'LAST- 1
	 loop
	   INV(i,j) := Pol(j + 1);
	 end loop;
       else
	 INV(i,1) := -Pol(1)/x(i);
	 for j in x'FIRST + 1..x'LAST - 1
	 loop
	    INV(i,j) := (INV(i,j - 1) - Pol(j))/x(i);
	 end loop;
       end if;
       for j in x'range
       loop
	 INV(i,j) := INV(i,j)/Di;
       end loop;
    end loop;
    return INV;
end INVERSE_VAN_DER_MONDE;
-- 
procedure DECOMPOSITION_QR(A: in MATRICE; Q, R: in out MATRICE) is
c, s: FLOAT;

procedure CALCUL_ROTATION(A: in MATRICE; q, p: in INTEGER;
			  c, s: in out FLOAT) is
EPS: constant FLOAT := 1.0E-12;
b, t: FLOAT;
begin
    if (abs(A(q,p)) < EPS) then
       c := 1.0;
       s := 0.0;
    else
       if (A(p,p) < EPS) then
	  c := 0.0;
	  s := 1.0;
       else
	  b := A(p,p)/A(q,p);
	  t := MATH1.SQRT(1.0 + b*b) + abs(b);
	  t := t*t;
	  c := (t - 1.0)/(t + 1.0);
	  s := c/b;
       end if;
    end if;
end CALCUL_ROTATION;

procedure PRODUIT_GA(q, p: in INTEGER; c, s: in FLOAT; A: in out MATRICE) is
Aux: FLOAT;
begin
   for j in A'range(2)
   loop
      Aux := c*A(p,j) + s*A(q,j);
      A(q,j) := -s*A(p,j) + c*A(q,j);
      A(p,j) := Aux;
   end loop;
end PRODUIT_GA;
begin
    Q := (others => (others => 0.0));
    for i in A'range(1)
    loop
       Q(i,i) := 1.0;
    end loop;
    declare
       AA: MATRICE(A'range(1),A'range(2)) := A;
    begin
       for p in A'FIRST(1)..A'LAST(1) - 1
       loop
	  for i in p + 1..A'LAST(1)
	  loop
	     CALCUL_ROTATION(AA,i,p,c,s);
	     PRODUIT_GA(i,p,c,s,AA);
	     PRODUIT_GA(i,p,c,s,Q);
	  end loop;
       end loop;
       R := AA;
       Q := TRANSPOSE(Q);
    end;
end DECOMPOSITION_QR;
-- 
function TEST_DIAGONALE_DOMINANTE(A: in MATRICE) return BOOLEAN is
S: FLOAT;
begin
   for i in A'range(1)
   loop
      S := 0.0;
      for j in A'range(2)
      loop
	 if (j /= i) then
	    S := S + abs(A(i,j));
	 end if;
      end loop;
      if (S > abs(A(i,i))) then
	 return FALSE;
      end if;
   end loop;
   return TRUE;
end TEST_DIAGONALE_DOMINANTE;
-- 
procedure JACOBI(A: in MATRICE; b: in VECTEUR; x: in out VECTEUR) is
MAX_ITERATION: constant NATURAL := 500;
EPS: constant FLOAT := 1.0E-12;
ITERATION: NATURAL := 0;
CONTINUER: BOOLEAN := TRUE;
S: FLOAT;
v: VECTEUR(b'range);
begin
    if (not TEST_DIAGONALE_DOMINANTE(A)) then
       PUT_LINE("Erreur dans JACOBI: A n'est pas a diagonale strictement dominante");
       raise ERREUR_DIAGONALE_NON_DOMINANTE;
    end if;
    for i in b'range
    loop
       x(i) := b(i)/A(i,i);
    end loop;
    while (ITERATION < MAX_ITERATION) and (CONTINUER)
    loop
       ITERATION := ITERATION + 1;
       for i in A'range(1)
       loop
	  S := b(i);
	  for j in A'range(2)
	  loop
	      if (j /= i) then
		 S := S - A(i,j)*x(j);
	      end if;
	   end loop;
	   v(i) := S/A(i,i);
       end loop;
       CONTINUER := (NORME_DU_SUP(v - x) > EPS);
       x := v;
    end loop;
    if (not CONTINUER) then
       TEXT_IO.NEW_LINE(IMP);
       TEXT_IO.PUT(IMP,"Nombre d'iterations: "); IIO.PUT(IMP,ITERATION);
       TEXT_IO.NEW_LINE(IMP);
    else
       PUT_LINE("Erreur dans JACOBI: Convergence trop lente ou divergence");
       raise ERREUR_NON_CONVERGENCE;
    end if;
end JACOBI;
-- 
procedure RELAXATION(A: in MATRICE; b: in VECTEUR; OMEGA: in FLOAT := 1.0;
		 x: in out VECTEUR) is
MAX_ITERATION: constant NATURAL := 500;
EPS: constant FLOAT := 1.0E-12;
RESIDU, NORME_RESIDU: FLOAT;
ITERATION: NATURAL := 0;
CONTINUER: BOOLEAN := TRUE;
begin
    if (not TEST_DIAGONALE_DOMINANTE(A)) then
       PUT_LINE("Erreur dans RELAXATION: A n'est pas a diagonale strictement dominante");
       raise ERREUR_DIAGONALE_NON_DOMINANTE;
    end if;
    for i in b'range
    loop
       x(i) := b(i)/A(i,i);
    end loop;
    while (ITERATION < MAX_ITERATION) and (CONTINUER)
    loop
       ITERATION := ITERATION + 1;
       NORME_RESIDU := 0.0;
       for i in A'range(1)
       loop
	  RESIDU := -b(i);
	  for j in A'range(2)
	  loop
	     RESIDU := RESIDU + A(i,j)*x(j);
	  end loop;
	  RESIDU := RESIDU/A(i,i);
	  x(i) := x(i) - OMEGA*RESIDU;
	  if (abs(RESIDU) > NORME_RESIDU) then
	     NORME_RESIDU := abs(RESIDU);
	  end if;
       end loop;
       CONTINUER := (NORME_RESIDU > EPS);
    end loop;
    if (not CONTINUER) then
       TEXT_IO.NEW_LINE(IMP);
       TEXT_IO.PUT(IMP,"Pour Omega = "); FIO.PUT(IMP,OMEGA,1,3,0);
       TEXT_IO.NEW_LINE(IMP);
       TEXT_IO.PUT(IMP,"Nombre d'iterations: "); IIO.PUT(IMP,ITERATION);
       TEXT_IO.NEW_LINE(IMP);
    else
       PUT_LINE("Erreur dans RELAXATION: Convergence trop lente ou divergence");
       raise ERREUR_NON_CONVERGENCE;
    end if;
end RELAXATION;
-- 

end ALGEBLIN;

