with MATH1, MATH3, MATRIX, DONNEES_POINTS;
use MATH1, MATH3, MATRIX, DONNEES_POINTS;

package body BEZIER is

function POLYNOME_BERNSTEIN(P: in VECTEUR; t: in FLOAT) return FLOAT is
-- Calcul du polynome de Bernstein par l'algorithme de De Casteljeau.
Q: VECTEUR(P'range) := P;
begin
    for j in 1..P'LAST loop
       for i in 0..P'LAST - j loop
	  Q(i) := (1.0 - t)*Q(i) + t*Q(i + 1);
       end loop;
    end loop;
    return Q(0);
end POLYNOME_BERNSTEIN;
-- 
procedure CALCUL_BEZIER(x, y: in VECTEUR; t: in FLOAT; Bx, By: out FLOAT) is
-- Calcul du polynome de Bezier par l'algorithme de De Casteljeau.
x2: VECTEUR(x'range) := x;
y2: VECTEUR(y'range) := y;
begin
    for j in 1..x'LAST loop
       for i in 0..x'LAST - j loop
	  x2(i) := (1.0 - t)*x2(i) + t*x2(i + 1);
	  y2(i) := (1.0 - t)*y2(i) + t*y2(i + 1);
       end loop;
    end loop;
    Bx := x2(0);
    By := y2(0);
end CALCUL_BEZIER;
-- 
procedure TRACE_BASE_BERNSTEIN(n: in NATURAL) is
-- Graphes des fonctions de base de Bernstein, Bnk, pour k = 0, 1, ..., n.
-- Utilisation de l'algorithme non recursif de De Casteljeau.
NB_POINTS: constant NATURAL := 300;
P: VECTEUR(0..n);
t, Pas: FLOAT;
begin
    Pas := 1.0/FLOAT(NB_POINTS);
    INIT_GRAPHIQUE;
    FENETRE_GRAPHIQUE(-0.2,1.2,-0.1,1.2);
    for k in 0..n loop
       P := (others => 0.0);
       P(k) := 1.0;
       t := 0.0;
       DEPLACE(t,POLYNOME_BERNSTEIN(P,t));
       for i in 1..NB_POINTS - 1 loop
	  t := t + Pas;
	  TRACE(t,POLYNOME_BERNSTEIN(P,t));
       end loop;
    end loop;
    XY_AXES(0.0,0.0,0.2,0.2,TRUE);
    SORTIE_GRAPHIQUE;
end TRACE_BASE_BERNSTEIN;
-- 
procedure COURBE_BEZIER(x, y: in VECTEUR; C: in out COURBE) is
Pas, t: FLOAT;
begin
    EXTREMA_POINTS(x,y,C.uMin,C.uMax,C.vMin,C.vMax);
    t := 0.0;
    Pas := 1.0/FLOAT(C.n);
    for i in 0..C.n loop
       CALCUL_BEZIER(x,y,t,C.u(i),C.v(i));
       C.uMin := MIN(C.uMin,C.u(i));
       C.uMax := MAX(C.uMax,C.u(i));
       C.vMin := MIN(C.vMin,C.v(i));
       C.vMax := MAX(C.vMax,C.v(i));
       t := t + Pas;
    end loop;
end COURBE_BEZIER;
-- 

end BEZIER;
