-- CALCUL DU COEFFICIENT DE TRAINEE EN FONCTION DU NOMBRE DE REYNOLDS
--                     CAS D'UN CORPS SPHERIQUE

with TEXT_IO, IIO, FIO, CRT, COMMON_MATH0, MATH0, MATH1, COMMON_MATH3, MATH3,
     CURVE;
use TEXT_IO, IIO, FIO, CRT, COMMON_MATH0, MATH0, MATH1, COMMON_MATH3, MATH3,
    CURVE;

procedure BILLE is

NBR_INTERVALLES: constant NATURAL := 7;
			      -- Nombre maximale d'intervalles dans [0,50000]
			      -- Dans chaque intervalle on a des mesures
			      -- experimentales de Re et Cd

NBR_MESURES: constant NATURAL := 6;
			      -- Nombre maximale de mesure des Re et Cd dans
			      -- chaque intervalle

type MATRICE_MESURES is array(POSITIVE range 1..NBR_INTERVALLES,
			   POSITIVE range 1..NBR_MESURES) of FLOAT;
     -- Dans l'intervalle N j, les mesures de Re et Cd sont notees
     -- Re(j,i) et Cd(j,i) ou 1 <= i <= NBR_MESURES

type VECTEUR_COEFF is array(POSITIVE range 1..NBR_INTERVALLES,
			 NATURAL range 0..2) of FLOAT;
     -- Dans l'intervalle N j, les coefficients de Morsi sont notes
     --  K(j,i), ou i = 1, 2, 3.

type VECTEUR_ENTIER is array(POSITIVE range 1..NBR_INTERVALLES) of NATURAL;
     -- Dans l'intervalle N j, le nombre de mesures est note n(j)

Re, Cd: MATRICE_MESURES;
nn: VECTEUR_ENTIER;
K: VECTEUR_COEFF;
Rho, Rhof, g, Nu, d, t0, tMax, z0, v0, A, B, C: FLOAT;

procedure DONNEES_DU_FICHIER(t0, tMax, z0, v0, Rho, Rhof, Nu, g, d,
			     A, B, C: in out FLOAT;
			     nn: in out VECTEUR_ENTIER;
			     Re, Cd: out MATRICE_MESURES;
                             K: out VECTEUR_COEFF) is
Nom: STRING(1..50);
FichierDonnees: FILE_TYPE;
RhoBarre: FLOAT;
begin
    PUT_LINE("   Donnees du fichier");
    LIRE_FICHIER(FichierDonnees,Nom);
    GET(FichierDonnees,t0);
    GET(FichierDonnees,tMax);
    GET(FichierDonnees,z0);
    GET(FichierDonnees,v0);
    GET(FichierDonnees,Rho);
    GET(FichierDonnees,Rhof);
    GET(FichierDonnees,Nu);
    GET(FichierDonnees,g);
    GET(FichierDonnees,d);
    SKIP_LINE(FichierDonnees);
    for j in 1..NBR_INTERVALLES loop
       GET(FichierDonnees,nn(j));
       SKIP_LINE(FichierDonnees);
       for i in 1..nn(j) loop
	  GET(FichierDonnees,Re(j,i));
       end loop;
       SKIP_LINE(FichierDonnees);
       for i in 1..nn(j) loop
	  GET(FichierDonnees,Cd(j,i));
       end loop;
       SKIP_LINE(FichierDonnees);
    end loop;
    for i in 1..NBR_INTERVALLES loop
       for j in 0..2 loop
	  GET(FichierDonnees,K(i,j));
       end loop;
       SKIP_LINE(FichierDonnees);
    end loop;
    CLOSE(FichierDonnees);
    RhoBarre := Rhof/Rho;
    A := 1.0 + 0.5*RhoBarre;
    B := g*(1.0 - RhoBarre);
    C := 0.75*RhoBarre/d;
end DONNEES_DU_FICHIER;

procedure AFFICHAGE_DONNEES is
begin
    CLRSCR;
    PUT_LINE(IMP,"            Donnees du probleme");
    NEW_LINE(IMP);
    PUT_LINE(IMP,"Conditions initiales:");
    PUT(IMP,"                      t0 = ");
    PUT(IMP,t0,6,3,0);
    NEW_LINE(IMP);
    PUT(IMP,"                      tMax = ");
    PUT(IMP,tMax,6,3,0);
    NEW_LINE(IMP);
    PUT(IMP,"                      z0 = ");
    PUT(IMP,z0,6,3,0);
    NEW_LINE(IMP);
    PUT(IMP,"                      v0 = ");
    PUT(IMP,v0,6,3,0);
    NEW_LINE(IMP);
    NEW_LINE(IMP);
    PUT_LINE(IMP,"Caracteristtiques de la bille et du fluide:");
    NEW_LINE(IMP);
    PUT(IMP,"                      Rho = ");
    PUT(IMP,Rho,6,3,0);
    NEW_LINE(IMP);
    PUT(IMP,"                      Rhof = ");
    PUT(IMP,Rhof,6,3,0);
    NEW_LINE(IMP);
    PUT(IMP,"                      Nu = ");
    PUT(IMP,Nu,6,6,0);
    NEW_LINE(IMP);
    PUT(IMP,"                      g = ");
    PUT(IMP,g,6,3,0);
    NEW_LINE(IMP);
    PUT(IMP,"                      d = ");
    PUT(IMP,d,6,3,0);
    NEW_LINE(IMP);
    PAUSE;
    CLRSCR;
end AFFICHAGE_DONNEES;

function Cd_j(K: in VECTEUR_COEFF; j: in INTEGER; x: in FLOAT)
	 return FLOAT is
-- Evaluation du polynome definissant la fonction Cd sur l'intervalle N j
begin
    return K(j,0) + K(j,1)*x + K(j,2)*x*x;
end Cd_j;

function F(v: in FLOAT) return FLOAT is
r, C_d: FLOAT;
j: INTEGER;
begin
    r := v*d/Nu;
    if r = 0.0 then
       return (B - C*K(1,2)*Nu/(d*d))/A;
    else
       j := 0;
       loop
          j := j + 1;
          exit when (r >= Re(j,1)) and (r < Re(j,nn(j)));
       end loop;
       C_d := Cd_j(K,j,1.0/r);
       return (B - C*v*ABS(v)*C_d)/A;
    end if;
end F;

procedure RK4(AFFICHER: in BOOLEAN; Cz, Cv: in out COURBE) is
Pas: FLOAT;
z_k1, z_k2, z_k3, z_k4, v_k1, v_k2, v_k3, v_k4: FLOAT;
begin
    Cz.uMin := t0;
    Cz.uMax := tMax;
    Cv.uMin := t0;
    Cv.uMax := tMax;
    Cz.vMin := 1.0E+38;
    Cz.vMax := -1.0E+38;
    Cv.vMin := 1.0E+38;
    Cv.vMax := -1.0E+38;
    Cz.u(0) := t0;
    Cv.u(0) := t0;
    Cz.v(0) := z0;
    Cv.v(0) := v0;
    Pas := (tMax - t0)/FLOAT(Cz.n);
    for i in 1..Cz.n
    loop
       Cz.u(i) := Cz.u(i - 1) + Pas;
       Cv.u(i) := Cz.u(i);
       z_k1 := Pas*Cv.v(i - 1);
       v_k1 := Pas*F(Cv.v(i - 1));
       z_k2 := Pas*(Cv.v(i - 1) + 0.5*v_k1);
       v_k2 := Pas*F(Cv.v(i - 1) + 0.5*v_k1);
       z_k3 := Pas*(Cv.v(i - 1) + 0.5*v_k2);
       v_k3 := Pas*F(Cv.v(i - 1) + 0.5*v_k2);
       z_k4 := Pas*(Cv.v(i - 1) + v_k3);
       v_k4 := Pas*F(Cv.v(i - 1) + v_k3);
       Cz.v(i) := Cz.v(i - 1) + (z_k1 + 2.0*z_k2 + 2.0*z_k3 + z_k4)/6.0;
       Cz.vMin := MIN(Cz.vMin,Cz.v(i));
       Cz.vMax := MAX(Cz.vMax,Cz.v(i));
       Cv.v(i) := Cv.v(i - 1) + (v_k1 + 2.0*v_k2 + 2.0*v_k3 + v_k4)/6.0;
       Cv.vMin := MIN(Cv.vMin,Cv.v(i));
       Cv.vMax := MAX(Cv.vMax,Cv.v(i));
       if (AFFICHER) then
          PUT(IMP,"t = ");
          PUT(IMP,Cz.u(i),6,3,0);
          PUT(IMP,"   z = ");
          PUT(IMP,Cz.v(i),6,3,0);
          PUT(IMP,"   v = ");
          PUT(IMP,Cv.v(i),6,3,0);
          NEW_LINE(IMP);
          if (i mod 20 = 0) then
             PAUSE;
          end if;
       end if;
    end loop;
end RK4;

procedure TRACE_POSITION_VITESSE is
tFenMin, tFenMax, zFenMin, zFenMax, vFenMin, vFenMax,
tUnite, zUnite, vUnite: FLOAT;
Afficher: BOOLEAN;
Cz, Cv: COURBE(200);
begin
    Afficher := LIRE_REPONSE("Voulez vous l'affichage des resultats? ");
    RK4(AFFICHER,Cz,Cv);
    tUnite := (tMax - t0)/20.0;
    zUnite := (Cz.vMax - Cz.vMin)/20.0;
    vUnite := (Cv.vMax - Cv.vMin)/20.0;
    tFenMin := Cz.uMin - tUnite;
    tFenMax := Cz.uMax + tUnite;
    zFenMin := Cz.vMin - zUnite;
    zFenMax := Cz.vMax + zUnite;
    vFenMin := Cv.vMin - vUnite;
    vFenMax := Cv.vMax + vUnite;
    INIT_GRAPHIQUE;
    FENETRE(tFenMin,tFenMax,zFenMin,zFenMax);
    CADRE_GRAPHIQUE(X_MINIMUM,X_MINIMUM + ((X_MAXIMUM - X_MINIMUM)/2) - 1,
                    Y_MINIMUM + MARGE,Y_MAXIMUM - MARGE);
    DEPLACE(Cz.u(0),Cz.v(0));
    for i in 1..Cz.n
    loop
       TRACE(Cz.u(i),Cz.v(i));
    end loop;
    XY_AXES(0.0,0.0,tUnite,zUnite,TRUE);
    FENETRE(tFenMin,tFenMax,vFenMin,vFenMax);
    CADRE_GRAPHIQUE(X_MINIMUM + ((X_MAXIMUM - X_MINIMUM)/2) + 1,X_MAXIMUM,
                    Y_MINIMUM + MARGE, Y_MAXIMUM - MARGE);
    DEPLACE(Cv.u(0),Cv.v(0));
    for i in 1..Cv.n
    loop
        TRACE(Cv.u(i),Cv.v(i));
    end loop;
    XY_AXES(0.0,0.0,tUnite,vUnite,TRUE);
    TITRE("Graphe de z(t)                              Graphe de v(t)");
    SORTIE_GRAPHIQUE;
end TRACE_POSITION_VITESSE;

begin
    MODE_AFFICHAGE;
    DONNEES_DU_FICHIER(t0,tMax,z0,v0,Rho,Rhof,Nu,g,d,A,B,C,nn,Re,Cd,K);
    AFFICHAGE_DONNEES;
    PUT_LINE("Resolution du systeme differentiel donnant z(t) et v(t)");
    NEW_LINE;
    TRACE_POSITION_VITESSE;
    CLOSE(IMP);
end BILLE;

