-- CALCUL DU COEFFICIENT DE TRAINEE EN FONCTION DU NOMBRE DE REYNOLDS
--                     CAS D'UN CORPS SPHERIQUE

with TEXT_IO, IIO, FIO, CRT, COMMON_MATH0, MATH0, MATH1, MATH3, CURVE;

procedure BILLE1 is

use TEXT_IO, IIO, FIO, CRT, COMMON_MATH0, MATH0, MATH1, MATH3, CURVE;

RE_MAX: constant FLOAT := 50000.0;
			      -- Valeur maximale des nombres de Reynolds Re
NBR_INTERVALLES: constant NATURAL := 7;
			      -- Nombre maximale d'intervalles dans [0,50000]
			      -- Dans chaque intervalle on a des mesures
			      -- experimentales de Re et Cd

NBR_MESURES: constant NATURAL := 6;
			      -- Nombre maximale de mesure des Re et Cd dans
			      -- chaque intervalle

type MATRICE_MESURES is array(POSITIVE range 1..NBR_INTERVALLES,
			   POSITIVE range 1..NBR_MESURES) of FLOAT;
     -- Dans l'intervalle N j, les mesures de Re et Cd sont notees
     -- Re(j,i) et Cd(j,i) ou 1 <= i <= NBR_MESURES

type VECTEUR_COEFF is array(POSITIVE range 1..NBR_INTERVALLES,
			 NATURAL range 0..2) of FLOAT;
     -- Dans l'intervalle N j, les coefficients de Morsi sont notes
     --  K(j,i), ou i = 1, 2, 3.

type VECTEUR_ENTIER is array(POSITIVE range 1..NBR_INTERVALLES) of NATURAL;
     -- Dans l'intervalle N j, le nombre de mesures est note n(j)

type VECTEUR is array(POSITIVE range 1..NBR_MESURES) of FLOAT;
type VECTEUR_3 is array(POSITIVE range 1..3) of FLOAT;

Re, Cd: MATRICE_MESURES;
nn: VECTEUR_ENTIER;
K: VECTEUR_COEFF;
Rho, Rhof, g, Nu, d, h, t0, tMax, z0, v0: FLOAT;

procedure DONNEES_DU_FICHIER(t0,tMax,z0,v0,Rho,Rhof,Nu,g,d: out FLOAT;
			     nn: in out VECTEUR_ENTIER;
			     Re, Cd: out MATRICE_MESURES) is
Nom: STRING(1..50);
FichierDonnees: FILE_TYPE;
begin
    PUT_LINE("   Donnees du fichier");
    LIRE_FICHIER(FichierDonnees,Nom);
    GET(FichierDonnees,t0);
    GET(FichierDonnees,tMax);
    GET(FichierDonnees,z0);
    GET(FichierDonnees,v0);
    GET(FichierDonnees,Rho);
    GET(FichierDonnees,Rhof);
    GET(FichierDonnees,Nu);
    GET(FichierDonnees,g);
    GET(FichierDonnees,d);
    SKIP_LINE(FichierDonnees);
    for j in 1..NBR_INTERVALLES loop
       GET(FichierDonnees,nn(j));
       SKIP_LINE(FichierDonnees);
       for i in 1..nn(j) loop
	  GET(FichierDonnees,Re(j,i));
       end loop;
       SKIP_LINE(FichierDonnees);
       for i in 1..nn(j) loop
	  GET(FichierDonnees,Cd(j,i));
       end loop;
       SKIP_LINE(FichierDonnees);
    end loop;
    CLOSE(FichierDonnees);
end DONNEES_DU_FICHIER;

procedure AFFICHAGE_DONNEES is
begin
    CLRSCR;
    PUT_LINE(IMP,"            Donnees du probleme");
    NEW_LINE(IMP);
    PUT_LINE(IMP,"Conditions initiales:");
    PUT(IMP,"                      t0 = ");
    PUT(IMP,t0,6,3,0);
    NEW_LINE(IMP);
    PUT(IMP,"                      tMax = ");
    PUT(IMP,tMax,6,3,0);
    NEW_LINE(IMP);
    PUT(IMP,"                      z0 = ");
    PUT(IMP,z0,6,3,0);
    NEW_LINE(IMP);
    PUT(IMP,"                      v0 = ");
    PUT(IMP,v0,6,3,0);
    NEW_LINE(IMP);
    NEW_LINE(IMP);
    PUT_LINE(IMP,"Caracteristtiques de la bille et du fluide:");
    NEW_LINE(IMP);
    PUT(IMP,"                      Rho = ");
    PUT(IMP,Rho,6,3,0);
    NEW_LINE(IMP);
    PUT(IMP,"                      Rhof = ");
    PUT(IMP,Rhof,6,3,0);
    NEW_LINE(IMP);
    PUT(IMP,"                      Nu = ");
    PUT(IMP,Nu,6,6,0);
    NEW_LINE(IMP);
    PUT(IMP,"                      g = ");
    PUT(IMP,g,6,3,0);
    NEW_LINE(IMP);
    PUT(IMP,"                      d = ");
    PUT(IMP,d,6,3,0);
    NEW_LINE(IMP);
    Pause;
    CLRSCR;
end AFFICHAGE_DONNEES;

function Cd_j(K: in VECTEUR_COEFF; j: in INTEGER; x: in FLOAT)
	 return FLOAT is
-- Evaluation du polynome definissant la fonction Cd sur l'intervalle N j
begin
    return K(j,0) + K(j,1)*x + K(j,2)*x*x;
end Cd_j;

procedure EQUATIONS_NORMALES(n: in INTEGER; u, v: in VECTEUR;
		       a, b, c, d: in out FLOAT; Gamma: in out VECTEUR_3) is
-- Formation du systeme lineaire, dont les coefficients w1, w2, w3 du
-- polynome de degre 2 qui passe au plus pres du nuage de points defini
-- par u et v, sont solutions (regression polynomiale).
-- La matrice et le second membre sont donnes par:
--
--                       n  a  b              Gamma1
--                   A = a  b  c, et Gamma =  Gamma2
--                       b  c  d              Gamma3
--
-- avec a = u1 + ... + un, b = u1**2 + ... + un**2,
--      c = u1**3 + ... + un**3
--      d = u1**4 + ... + un**4
--      Gamma1 = v1 + ... + vn, Gamma2 = u1*v1 + ... + un*vn
--      Gamma3 = u1**2*v1 + ... + un**2*vn

e: FLOAT;
begin
    a := u(1); b := a*u(1); c := b*u(1); d := c*u(1);
    Gamma(1) := v(1);
    Gamma(2) := Gamma(1)*u(1);
    Gamma(3) := Gamma(2)*u(1);
    for k in 2..n loop
	e := u(k);
        a := a + e;
	e := e*u(k);
        b := b + e;
	e := e*u(k);
        c := c + e;
	e := e*u(k);
        d := d + e;
	e := v(k);
	Gamma(1) := Gamma(1) + e;
	e := e*u(k);
	Gamma(2) := Gamma(2) + e;
	e := e*u(k);
	Gamma(3) := Gamma(3) + e;
    end loop;
end EQUATIONS_NORMALES;

procedure RESOLUTION_EQU_NORMALES(n: in INTEGER; a, b, c, d: in FLOAT;
			     Gamma: in out VECTEUR_3) is
-- Resolution du systeme d'equations normales donnant les coefficients du
-- polynome de regression, de degre 2, definit par u et v.
DD, D1, D2, D3: FLOAT;
begin
    DD := FLOAT(n)*(b*d - c*c) + 2.0*a*b*c - b*b*b - a*a*d;
    D1 := Gamma(1)*(b*d - c*c) + Gamma(2)*(b*c - a*d) + Gamma(3)*(a*c - b*b);
    D2 := Gamma(1)*(b*c - a*d) + Gamma(2)*(FLOAT(n)*d - b*b) +
	  Gamma(3)*(a*b - FLOAT(n)*c);
    D3 := Gamma(1)*(a*c - b*b) + Gamma(2)*(a*b - FLOAT(n)*c) +
	  Gamma(3)*(FLOAT(n)*b - a*a);
    Gamma(1) := D1/DD;
    Gamma(2) := D2/DD;
    Gamma(3) := D3/DD;
end RESOLUTION_EQU_NORMALES;

procedure CALCUL_COEFF_MORSI(Re, Cd: in MATRICE_MESURES;
			     K: out VECTEUR_COEFF) is
-- Calcul des coefficients K(j,i) de Morsi
a, b, c, d: FLOAT;
Gamma: VECTEUR_3;
u, v: VECTEUR;
begin
    for j in 1..NBR_INTERVALLES loop
       for i in 1..nn(j) loop
	    u(i) := 1.0/Re(j,i);
	    v(i) := Cd(j,i);
       end loop;
       EQUATIONS_NORMALES(nn(j),u,v,a,b,c,d,Gamma);
       RESOLUTION_EQU_NORMALES(nn(j),a,b,c,d,Gamma);
       for i in 0..2 loop
	 K(j,i) := Gamma(i + 1);
       end loop;
    end loop;
end CALCUL_COEFF_MORSI;

procedure AFICCHE_COEFF_MORSI(Re, Cd: in MATRICE_MESURES;
			      K: in VECTEUR_COEFF) is
begin
    for j in 1..NBR_INTERVALLES loop
       PUT_LINE(IMP,"Intervalle Numero " & INTEGER'IMAGE(j));
       for i in 0..2 loop
	  PUT(IMP,"K(" & INTEGER'IMAGE(j) & "," & INTEGER'IMAGE(i) &
		       ") = ");
	  PUT(IMP,k(j,i),3,3,0);
	  NEW_LINE(IMP);
       end loop;
       PUT_LINE(IMP,"Re             Cd (Experimental)   Cd (Calcule)");
       for i in 1..nn(j) loop
	  PUT(IMP,Re(j,i),6,3,0);
	  PUT(IMP,"      ");
	  PUT(IMP,Cd(j,i),6,3,0);
	  PUT(IMP,"     ");
	  PUT(IMP,Cd_j(K,j,1.0/Re(j,i)),6,3,0);
	  NEW_LINE(IMP);
       end loop;
       Pause;
    end loop;
end AFICCHE_COEFF_MORSI;

procedure COURBE_CD(Re, Cd: in MATRICE_MESURES; K: in VECTEUR_COEFF;
		    CC: in out Courbe) is
-- Definition de la courbe Ln(Cd) en fonction de Ln(Re).
Pas, Pas_r, r: FLOAT;
i: INTEGER;
begin
    i := 0;
    r := 0.01;
    CC.uMin := Ln(r);
    CC.uMax := Ln(RE_MAX);
    CC.vMin := 1.0E+37;
    CC.vMax := -1.0E+37;
    CC.u(0) := CC.uMin;
    CC.v(0) := Ln(Cd_j(K,1,1.0/r));
    Pas := (CC.uMax - CC.uMin)/FLOAT(CC.n);
    Pas_r := Exp(Pas);
    r := r*Pas_r;
    for j in 1..NBR_INTERVALLES loop
	WHILE (r <= Re(j,nn(j))) loop
	     i := i + 1;
	     CC.u(i) := CC.u(i-1) + Pas;
	     CC.v(i) := Ln(Cd_j(K,j,1.0/r));
	     CC.vMin := Min(CC.vMin,CC.v(i));
	     CC.vMax := Max(CC.vMax,CC.v(i));
             r := r*Pas_r;
	end loop;
    end loop;
end COURBE_CD;

procedure TRACE_CD(Re, Cd: in MATRICE_MESURES) is
CC: Courbe(200);
begin
    COURBE_CD(Re,Cd,K,CC);
    Trace_Une_Courbe(CC);
    for j in 1..NBR_INTERVALLES loop
	for i in 1..nn(j) loop
	    CROIX(Ln(Re(j,i)),Ln(Cd(j,i)));
	end loop;
    end loop;
    SORTIE_GRAPHIQUE;
end TRACE_CD;

begin
    Mode_Affichage;
    DONNEES_DU_FICHIER(t0,tMax,z0,v0,Rho,Rhof,Nu,g,d,nn,Re,Cd);
    AFFICHAGE_DONNEES;
    PUT_LINE("Calcul des coefficients de regression");
    CALCUL_COEFF_MORSI(Re,Cd,K);
    AFICCHE_COEFF_MORSI(Re,Cd,K);
    CLRSCR;
    PUT_LINE(" Trace de la courbe Ln(Cd) en fonction de Ln(Re)");
    PUT_LINE(" Patience je calcule ...");
    TRACE_CD(Re,Cd);
    CLOSE(IMP);
end BILLE1;


