with MATH0, MATH1, MATH3, MATRIX, DONNEES_POINTS;
use MATH0, MATH1, MATH3, MATRIX, DONNEES_POINTS;

package body B_SPLINE is

procedure NOEUDS(n, j: in NATURAL; Ne: out VECTEUR_NODAL) is
-- Calcul du vecteur nodal Ne
begin
    for i in 0..j - 1 loop
       Ne(i) := 0;
    end loop;
    for i in j..n loop
       Ne(i) := i - j + 1;
    end loop;
    for i in n + 1..n + j loop
       Ne(i) := n - j + 2;
    end loop;
end NOEUDS;
-- 
function BASE_BSPLINE(i, j: NATURAL; Ne: VECTEUR_NODAL; t: FLOAT)
	 return FLOAT is
-- Fonction Nij(t), pour i = 0, 1, ..., n et j = 1, ..., n + 1.
-- Definition recursive
S1, S2, Aux1, Aux2: FLOAT;
begin
   if (t < FLOAT(Ne(i))) or (FLOAT(Ne(i + j)) <= t) then
      return 0.0;
   else
      if j = 1 then
	 return 1.0; -- Puisque t est dans [Ne(i),Ne(i+1))
      else
	 Aux1 := BASE_BSPLINE(i,j-1,Ne,t);
	 if (Ne(i + j - 1) = Ne(i)) or (Aux1 = 0.0) then
	    S1 := 0.0;
	 else
	    S1 := (t - FLOAT(Ne(i)))/(FLOAT(Ne(i+j-1)) - FLOAT(Ne(i)))*Aux1;
	 end if;
	 Aux2 := BASE_BSPLINE(i + 1,j - 1,Ne,t);
	 if (Ne(i+j) = Ne(i+1)) or (Aux2 = 0.0) then
	    S2 := 0.0;
	 else
	    S2 := (FLOAT(Ne(i+j)) - t)/(FLOAT(Ne(i+j)) - FLOAT(Ne(i+1)))*Aux2;
	 end if;
	 return S1 + S2;
      end if;
   end if;
end BASE_BSPLINE;
-- 
procedure CALCUL_BSPLINE(j: in NATURAL; Ne: in VECTEUR_NODAL; x, y: in VECTEUR;
		   t: in FLOAT; Bx, By: out FLOAT) is
x1, y1, x2, y2: VECTEUR(x'range);
k: INTEGER;
n: NATURAL := x'LAST;
m, Sx1, Sx2, Sy1, Sy2: FLOAT;
begin
    for i in x'range loop
      x2(i) := x(i);
    end loop;
    for i in y'range loop
      y2(i) := y(i);
    end loop;
    if (t >= FLOAT(n - j + 2)) then
       Bx := x2(n);
       By := y2(n);
    else
       k := (j - 1) + TRUNC(t);
       for i in k - j + 1..k loop
	  x1(i) := x2(i);
	  y1(i) := y2(i);
       end loop;
       for p in reverse 2..j loop
	  for i in k - p + 1..k loop
	     if (Ne(i + p - 1) = Ne(i)) then
		Sx1 := 0.0;
		Sy1 := 0.0;
		Sx2 := 0.0;
		Sy2 := 0.0;
	      else
		m := (t - FLOAT(Ne(i)))/FLOAT(Ne(i + p - 1) - Ne(i));
		Sx1 := m*x1(i);
		Sy1 := m*y1(i);
		m := (FLOAT(Ne(i + p - 1)) - t)/FLOAT(Ne(i + p - 1) - Ne(i));
		Sx2 := m*x1(i - 1);
		Sy2 := m*y1(i - 1);
	      end if;
	      x2(i) := Sx1 + Sx2;
	      y2(i) := Sy1 + Sy2;
	   end loop;
	   for i in k - p + 1..k loop
	      x1(i) := x2(i);
	      y1(i) := y2(i);
	   end loop;
       end loop;
       Bx := x2(k);
       By := y2(k);
    end if;
end CALCUL_BSPLINE;
-- 
procedure COURBE_BSPLINE(j: in NATURAL; x, y: in VECTEUR; C: in out COURBE) is
-- Definition d'une courbe B_Spline definit par un polygone a n cotes,
-- P = { (xi,yi); i = 0, 1, ..., n } et par l'ordre j.
Pas, t: FLOAT;
n: NATURAL := x'LAST;
tMax: NATURAL := n - j + 2;
Ne: VECTEUR_NODAL(0..n+j);

begin
    EXTREMA_POINTS(x,y,C.uMin,C.uMax,C.vMin,C.vMax);
    NOEUDS(n,j,Ne);
    Pas := FLOAT(tMax)/FLOAT(C.n);
    t := 0.0;
    for i in 0..C.n loop
	CALCUL_BSPLINE(j,Ne,x,y,t,C.u(i),C.v(i));
	C.uMin := MIN(C.uMin,C.u(i));
	C.uMax := MAX(C.uMax,C.u(i));
	C.vMin := MIN(C.vMin,C.v(i));
	C.vMax := MAX(C.vMax,C.v(i));
	t := t + Pas;
    end loop;
end COURBE_BSPLINE;
-- 
procedure TRACE_BASE_BSPLINE(n, j: in NATURAL) is
-- Graphes des fonctions de base B_Spline Nij(t), pour i = 1, ..., n,
-- l'ordre j etant donne.
NB_POINTS: constant NATURAL := 300;
xMin, xMax, yMin, yMax, xorigine, yorigine, xUnite, yUnite,
xFenMin, xFenMax, yFenMin, yFenMax, Pas, t: FLOAT;
tMax: NATURAL := n - j + 2;
Ne: VECTEUR_NODAL(0..n+j);
begin
    NOEUDS(n,j,Ne);
    Pas := FLOAT(tMax)/FLOAT(NB_POINTS);
    INIT_GRAPHIQUE;
    FENETRE_GRAPHIQUE(-1.0,FLOAT(tMax + 1),-0.2,1.2);
    for i in 0..n loop
       t := 0.0;
       DEPLACE(t,BASE_BSPLINE(i,j,Ne,t));
       FOR k in 1..NB_POINTS - 1 loop
	  t := t + Pas;
	  TRACE(t,BASE_BSPLINE(i,j,Ne,t));
       end loop;
    end loop;
    XY_AXES(0.0,0.0,1.0,0.2,TRUE);
    SORTIE_GRAPHIQUE;
end TRACE_BASE_BSPLINE;
-- 

end B_SPLINE;
