-- ͻ
--                   Portabilite de ce corps de paquetage.               
--  Ce corps de paquetage est utilisable avec OPEN_ADA.                  
--  On utilise le paquetage INTERRUPT, fourni avec  OPEN_ADA, qui permet 
--  d'utiliser les interruptions DOS.                                    
-- ͼ

with TEXT_IO, INTERRUPT;

package body CRT is

procedure CURSOR(B: in BOOLEAN) is
R: INTERRUPT.REGISTERS;
begin
   R.AX := 16#0100#; -- Fonction O1: Definit la presence du curseur
		     -- et sa forme
   if B then
      R.CX := 16#0607#; -- Curseur present
   else
      R.CX := 16#0100#; -- Curseur absent
   end if;
   INTERRUPT.VECTOR(16#10#,R); -- Interruption video 10h du BIOS.
end CURSOR;
--  
function WHERE_X return COLONNE_TEXTE is
R: INTERRUPT.REGISTERS;
begin
   R.AX := 16#0300#; -- Fonction O3h: retourne la position du cursur dans DX.
   R.BX := 16#0000#;
   INTERRUPT.VECTOR(16#10#,R); -- Interruption video 10h du BIOS.
   return (R.DX MOD 256); -- Octet de poids faible de R.DX
end WHERE_X;
--  
function WHERE_Y return LIGNE_TEXTE is
R: INTERRUPT.REGISTERS;
begin
   R.AX := 16#0300#;
   R.BX := 16#0000#;
   INTERRUPT.VECTOR(16#10#,R);
   return (R.DX/256); -- Octet de poids fort de R.DX
end WHERE_Y;
--  
procedure GOTOXY(C: in COLONNE_TEXTE; L: in LIGNE_TEXTE) is
R: INTERRUPT.REGISTERS;
begin
   R.AX := 16#0200#; -- Fonction 02h: positionne le curseur en X = DL, Y = DH
   R.BX := 16#0000#;
   R.DX := L*256 + C; -- L'octet de poids fort de DX contient le numero de
		      -- ligne et celui de poids faible le numero de colonne.
   INTERRUPT.VECTOR(16#10#,R); -- Interruption video 10h du BIOS.
end GOTOXY;
--  
procedure CLREOL is
WY: LIGNE_TEXTE;
WX: COLONNE_TEXTE;
R: INTERRUPT.REGISTERS;
begin
    WX := WHERE_X;
    WY := WHERE_Y;
    if (WX = 79) then
      null;
    else
      R.AX := 16#0920#; -- Fonction O9h: affiche un caractere blanc
      R.BX := 16#0000#;
      R.CX := 79 - WX;  -- Nombre de blancs a ecrire.
      INTERRUPT.VECTOR(16#10#,R);
    end if;
end CLREOL;
--  
procedure CLRSCR is
R: INTERRUPT.REGISTERS;
begin
    R.AX := 16#0003#; -- Fonction 00h: Choix du mode video CO80
    R.BX := 16#0000#;
    INTERRUPT.VECTOR(16#10#,R);
end CLRSCR;
--  
function READKEY return CHARACTER is
R: INTERRUPT.REGISTERS;
C: CHARACTER;
begin
   loop
      begin
	R.AX := 16#0000#; -- Fonction 00h: Lecture d'un caractere sans echo.
	INTERRUPT.VECTOR(16#16#,R); -- Interruption DOS 16h
	C := CHARACTER'VAL(R.AX MOD 256); -- AH contient le code du caractere
	exit;
      exception -- Caractere de code ASCII > 127.
	when CONSTRAINT_ERROR => PUT_CHAR(ASCII.BEL);
      end;
    end loop;
    return C;
end READKEY;
--  
PROCEDURE PUT_CHAR(C: in CHARACTER) is
R: INTERRUPT.REGISTERS;
begin
   R.AX := CHARACTER'POS(C);
   INTERRUPT.VECTOR(16#29#,R);
end PUT_CHAR;
--  
procedure PUT_STRING(Msg: in STRING) is
begin
    for i in Msg'range loop
      PUT_CHAR(Msg(i));
    end loop;
end PUT_STRING;
--  
procedure PUT_LINE_STRING(Msg: in STRING) is
begin
    PUT_STRING(Msg);
    TEXT_IO.NEW_LINE;
end PUT_LINE_STRING;
--  

begin
    CLRSCR;
    CURSOR(TRUE);
end CRT;
