with MATH1, COMMON_MATRIX, MATH2;
use MATH1, COMMON_MATRIX, MATH2;

package body DIRICHLET is

procedure SYSTEME_DIRICHLET(p, q, f: in FONCTION; a, b, ya, yb: in FLOAT;
			    T: in out MATRICE_TRIDIAGONALE;
			    v: in out VECTEUR) is
-- Formation du systeme tridiagonal dont y = (y1, ...,yn) est solution
Pas, h, x: FLOAT;
begin
    Pas := (b - a)/FLOAT(v'LENGTH + 1);
    h := Pas*Pas;
    x := a;
    for k in v'range
    loop
	x := x + Pas;
	T(k)(1) := 1.0/h - EVALUE(p,x)/(2.0*Pas);
	T(k)(2) := EVALUE(q,x) - 2.0/h;
	T(k)(3) := 1.0/h + EVALUE(p,x)/(2.0*Pas);
	v(k) := EVALUE(f,x);
    end loop;
    v(1) := v(1) - T(T'FIRST)(1)*ya;
    v(v'LAST) := v(v'LAST) - T(T'LAST)(3)*yb;
end SYSTEME_DIRICHLET;
-- 
procedure RESOLUTION_DIRICHLET(p, q, f: in FONCTION; a, b, ya, yb: in FLOAT;
			       C: in out COURBE) is
-- Resolution du systeme tridiagonal dont y = CC.v est solution, par la methode
-- de double balayage de Cholesky.
Pas, m: FLOAT;
T: MATRICE_TRIDIAGONALE(1..C.n - 1);
w: VECTEUR(1..C.n - 1);
begin
    SYSTEME_DIRICHLET(p,q,f,a,b,ya,yb,T,w);
    C.u(0) := a;
    C.u(C.n) := b;
    C.uMin := a;
    C.uMax := b;
    C.v(0) := ya;
    C.v(C.n) := yb;
    C.vMin := 1.0E+37;
    C.vMax := -1.0E+37;
    Pas := (b - a)/FLOAT(C.n);
    C.u(1) := a + Pas;
    for k in T'FIRST + 1..T'LAST
    loop
	m := T(k)(1)/T(k-1)(2);
	T(k)(2) := T(k)(2) - m*T(k-1)(3);
	w(k) := w(k) - m*w(k-1);
	C.u(k) := C.u(k - 1) + Pas;
    end loop;
    C.v(T'LAST) := w(T'LAST)/T(T'LAST)(2);
    for k in reverse T'FIRST..T'LAST - 1
    loop
       C.v(k) := (w(k) - T(k)(3)*C.v(k+1))/T(k)(2);
       C.vMin := MIN(C.vMin,C.v(k));
       C.vMax := MAX(C.vMax,C.v(k));
    end loop;
end RESOLUTION_DIRICHLET;
-- 
procedure RESOLUTION_POISSON(p, f: in FONCTION; y0, y1: in FLOAT;
			     C: in out COURBE) is
Pas, h, m: FLOAT;
a, w: VECTEUR(1..C.n - 1);
begin
    Pas := 1.0/FLOAT(C.n);
    h := Pas*Pas;
    C.u(0) := 0.0;
    C.u(C.n) := 1.0;
    C.uMin := 0.0;
    C.uMax := 1.0;
    C.v(0) := y0;
    C.v(C.n) := y1;
    C.vMin := y1;
    C.vMax := y1;
    for k in 1..C.n - 1
    loop
       a(k) := 2.0 - h*EVALUE(p,C.u(k));
       C.u(k) := C.u(k - 1) + Pas;
       w(k) := -h*EVALUE(f,C.u(k));
    end loop;
    w(1) := y0 + w(1);
    w(C.n - 1) := y1 + w(C.n - 1);
    for k in 2..C.n - 1
    loop
	m := -1.0/a(k-1);
	a(k) := a(k) + m;
	w(k) := w(k) - m*w(k-1);
    end loop;
    C.v(C.n - 1) := w(C.n - 1)/a(C.n - 1);
    C.vMin := MIN(C.vMin,C.v(C.n-1));
    C.vMax := MAX(C.vMax,C.v(C.n-1));
    for k in reverse 1..C.n - 2
    loop
       C.v(k) := (w(k) + C.v(k+1))/a(k);
       C.vMin := MIN(C.vMin,C.v(k));
       C.vMax := Max(C.vMax,C.v(k));
    end loop;
end RESOLUTION_POISSON;
-- 
end DIRICHLET;

