with TEXT_IO, IIO, FIO, COMMON_MATH0, MATH0, MATH2;
use TEXT_IO, IIO, FIO, COMMON_MATH0, MATH0, MATH2;

package body EDP_ELLIPTIQUES is

MAX_ITERATIONS: constant INTEGER := 1000;
Eps: constant FLOAT := 1.0E-6;

procedure INIT_ELLIPTIQUE(h: in FLOAT; g1, g2, g3, g4: in FONCTION;
			  U: in out MATRICE) is
x: FLOAT := 0.0;
begin
   for i in U'range(1)
   loop
      U(i,U'FIRST(2)) := EVALUE(g1,x);
      U(i,U'LAST(2)) := EVALUE(g2,x);
      x := x + h;
   end loop;
   x := 0.0;
   for j in U'range(2)
   loop
      U(U'FIRST(1),j) := EVALUE(g3,x);
      U(U'LAST(1),j) := EVALUE(g4,x);
      x := x + h;
   end loop;
   for i in U'FIRST(1) + 1..U'LAST(1) - 1
   loop
      for j in U'FIRST(2) + 1..U'LAST(2) - 1
      loop
	 U(i,j) := 0.25*(U(i,U'FIRST(2)) + U(i,U'LAST(2)) +
			 U(U'FIRST(1),j) + U(U'LAST(1),j));
      end loop;
    end loop;
end INIT_ELLIPTIQUE;
-- 
--  RESOLUTION DE L'EQUATION DE POISSON DELTA(U) = O

procedure RESOLUTION_POISSON(f, g1, g2, g3, g4: in FONCTION;
			     RHO_JACOBI, h: in FLOAT;
			     U: in out MATRICE) is
x, y, ff, Residu, omega, NormeResidu: FLOAT;
k: INTEGER;
begin
    INIT_ELLIPTIQUE(h, g1, g2, g3, g4, U);
    Omega := 1.0;
    k := 1;
    CHRONOMETRE(0);
    loop
	x := 0.0;
	NormeResidu := 0.0;
	for i in U'FIRST(1) + 1..U'LAST(1) - 1
	loop
	    y := 0.0;
	    x := x + h;
	    for j in U'FIRST(2) + 1..U'LAST(2) - 1
	    loop
		y := y + h;
		ff := EVALUE(f,x,y);
		ff := ff*h*h;
		if ( ((i + j) mod 2) = (k mod 2) ) then
		   Residu := U(i+1,j) + U(i-1,j) + U(i,j+1) + U(i,j-1)
			      - 4.0*U(i,j) - ff;
		   NormeResidu := NormeResidu + abs(Residu);
		   U(i,j) := U(i,j) + 0.25*Omega*Residu;
		end if;
	    end loop;
	end loop;
	if k = 1 then
	   Omega := 1.0/(1.0 - 0.5*(RHO_JACOBI**2));
	else
	   Omega := 1.0/(1.0 - 0.25*(RHO_JACOBI**2)*Omega);
	end if;
	PUT("Iteration Numero ");
	PUT(k,5);
	PUT("   NormeResidu = ");
	PUT(NormeResidu,6,6,0);
	NEW_LINE;
	k := k + 1;
	exit when (k = MAX_ITERATIONS) or (NormeResidu <= Eps);
    end loop;
    if (k = MAX_ITERATIONS) then
       PUT_LINE("Erreur dans RESOLUTION_POISSON: convergence trop lente.");
       raise ERREUR_CONVERGENCE;
    else
       CHRONOMETRE(1);
       PUT(IMP,"Nombre d'iterations: "); PUT(IMP,k - 1); NEW_LINE(IMP,2);
    end if;
end RESOLUTION_POISSON;
-- 
--   RESOLUTION D'UNE EQUATION ELLIPTIQUE GENERALE

PROCEDURE RESOLUTION_ELLIPTIQUE(
		  f, g1, g2, g3, g4, Alpha, Beta, Gama, Delt, Epsi: FONCTION;
		  Omega, h: in FLOAT;
		  U: in out MATRICE) is
k: INTEGER;
a, b, c, d, e, ff, x, y, Residu, NormeResidu: FLOAT;
BEGIN
    INIT_ELLIPTIQUE(h,g1,g2,g3,g4,U);
     k := 1;
     CHRONOMETRE(0);
     loop
	NormeResidu := 0.0;
	x := 0.0;
	for i in U'FIRST(1) + 1..U'LAST(1) - 1
	loop
	    y := 0.0;
	    x := x + h;
	    for j in U'FIRST(2) + 1..U'LAST(2) - 1
	    loop
		y := y + h;
		a := EVALUE(Alpha,x,y) + h*EVALUE(Beta,x,y)*0.5;
		b := EVALUE(Alpha,x,y) - h*EVALUE(Beta,x,y)*0.5;
		c := EVALUE(Gama,x,y) + h*EVALUE(Delt,x,y)*0.5;
		d := EVALUE(Gama,x,y) - h*EVALUE(Delt,x,y)*0.5;
     e := -2.0*(EVALUE(Alpha,x,y) + EVALUE(Gama,x,y)) + h*h*EVALUE(Epsi,x,y);
		ff := EVALUE(f,x,y);
		ff := ff*h*h;
		if ( ((i + j) MOD 2) = (k MOD 2) ) then
		    Residu := a*U(i+1,j) + b*U(i-1,j) + c*U(i,j+1) +
			      d*U(i,j-1) + e*U(i,j) - ff;
		    NormeResidu := NormeResidu + abs(Residu);
		    U(i,j) := U(i,j) - Omega*Residu/e;
		end if;
	    end loop;
	end loop;
	PUT("Iteration numero"); PUT(k,5);
	PUT(": NormeResidu = "); PUT(NormeResidu,6,6,0); NEW_LINE;
	k := k + 1;
	exit when (k = MAX_ITERATIONS) or (NormeResidu <= Eps);
     end loop;
    if (k = MAX_ITERATIONS) then
       PUT_LINE("Erreur dans RESOLUTION_POISSON: convergence trop lente.");
       raise ERREUR_CONVERGENCE;
    else
       CHRONOMETRE(1);
       PUT(IMP,"Nombre d'iterations: "); PUT(IMP,k - 1); NEW_LINE(IMP,2);
    end if;
end RESOLUTION_ELLIPTIQUE;
-- 
procedure AFFICHER_SOLUTION(U: in MATRICE) is
begin
    for j in reverse U'range(2)
    loop
       for i in U'range(1)
       loop
	 PUT(IMP,U(i,j),4,3,0); PUT(IMP," ");
       end loop;
       NEW_LINE(IMP);
       if (((U'LAST(2) - j) + 1) mod 10 = 0) then
	  PAUSE;
       end if;
    end loop;
end AFFICHER_SOLUTION;
-- 
-- RESOLUTION DE L'EQUATION DE POISSON DANS UN RECTANGLE TROUE

procedure INIT_ELLIPTIQUE_2(p, q, r, s: in INTEGER; h: in FLOAT;
			    g1, g2, g3, g4, h1, h2, h3, h4: in FONCTION;
			    U: in out MATRICE) is
x: FLOAT := 0.0;
begin
   for i in U'range(1)
   loop
      for j in U'range(2)
      loop
	 if (p < i) and (i < q) and (r < j) and (j < s) then
	    U(i,j) := 1.0E+37;
	 else
	    U(i,j) := 0.0;
	 end if;
      end loop;
   end loop;
   for i in U'range(1)
   loop
      U(i,U'FIRST(2)) := EVALUE(g1,x);
      if (p <= i) and (i <= q) then
	 U(i,r) := EVALUE(h1,x);
	 U(i,s) := EVALUE(h2,x);
      end if;
      U(i,U'LAST(2)) := EVALUE(g2,x);
      x := x + h;
   end loop;
   x := 0.0;
   for j in U'range(2)
   loop
      U(U'FIRST(1),j) := EVALUE(g3,x);
      if (r <= j) and (j <= s) then
	 U(p,j) := EVALUE(h3,x);
	 U(q,j) := EVALUE(h4,x);
      end if;
      U(U'LAST(1),j) := EVALUE(g4,x);
      x := x + h;
   end loop;
end INIT_ELLIPTIQUE_2;
-- 
procedure RESOLUTION_POISSON_2(f, g1, g2, g3, g4, h1, h2, h3, h4: in FONCTION;
			       p , q , r , s: in INTEGER;
			       RHO_JACOBI, h: in FLOAT;
			       U: in out MATRICE) is
k: INTEGER;
x, y, ff, Residu, Omega, NormeResidu: FLOAT;
begin
    INIT_ELLIPTIQUE_2(p,q,r,s,h,g1,g2,g3,g4,h1,h2,h3,h4,U);
    Omega := 1.0;
    k := 1;
    CHRONOMETRE(0);
    loop
       x := 0.0;
       NormeResidu := 0.0;
       for i in U'FIRST(1) + 1..U'LAST(1) - 1
       loop
	  y := 0.0;
	  x := x + h;
	  if (i < p) or (q < i) then
	     for j in U'FIRST(2) + 1..U'LAST(2) - 1
	     loop
		y := y + h;
		ff := EVALUE(f,x,y) ;
		ff := ff*h*h;
		if ( ((i + j) mod 2) = (k mod 2) ) then
		   Residu := U(i+1,j) + U(i-1,j) + U(i,j+1) + U(i,j-1)
			     - 4.0*U(i,j) - ff;
		   NormeResidu := NormeResidu + abs(Residu);
		   U(i,j) := U(i,j) + 0.25*Omega*Residu;
		end if;
	      end loop;
	   else
	      for j in U'FIRST(2) + 1..U'LAST(2) - 1
	      loop
		 y := y + h;
		 if (j < r) or (s < j) then
		    ff := EVALUE(f,x,y) ;
		    ff := ff*h*h;
		    if ( ((i + j) mod 2) = (k mod 2) ) then
		       Residu := U(i+1,j) + U(i-1,j) + U(i,j+1) + U(i,j-1)
				 - 4.0*U(i,j) - ff;
		       NormeResidu := NormeResidu + abs(Residu);
		       U(i,j) := U(i,j) + 0.25*Omega*Residu;
		    end if;
		 end if;
	      end loop;
	   end if;
       end loop;
       if k = 1 then
	  Omega := 1.0/(1.0 - 0.5*(RHO_JACOBI**2));
       else
	  Omega := 1.0/(1.0 - 0.25*(RHO_JACOBI**2)*Omega);
       end if;
       PUT("Iteration numero "); PUT(k,5);
       PUT(": NormeResidu = "); PUT(NormeResidu,6,6,0); NEW_LINE;
       k := k + 1;
       exit when (k = MAX_ITERATIONS) or (NormeResidu <= Eps);
    end loop;
    if (k = MAX_ITERATIONS) then
       PUT_LINE("Erreur dans RESOLUTION_POISSON: convergence trop lente.");
       raise ERREUR_CONVERGENCE;
    else
       CHRONOMETRE(1);
       PUT(IMP,"Nombre d'iterations: "); PUT(IMP,k - 1); NEW_LINE(IMP,2);
    end if;
end RESOLUTION_POISSON_2;
-- 
procedure AFFICHER_SOLUTION_2(p, q, r, s: in INTEGER; U: in MATRICE) is
begin
    for j in reverse U'range(2)
    loop
       for i in U'range(1)
       loop
	   if (r < j) and (j < s) and (p < i) and (i < q) then
	     PUT(IMP,"       ");
	   else
	     PUT(IMP,u(i,j),4,3,0); PUT(IMP," ");
	   end if;
	end loop;
	NEW_LINE(IMP);
	if (((U'LAST(2) - j) + 1) mod 10 = 0) then
	   PAUSE;
	end if;
    end loop;
end AFFICHER_SOLUTION_2;
-- 

end EDP_ELLIPTIQUES;


