with COMMON_MATH1, MATH1, MATH3, NOMBRES_COMPLEXES;
use COMMON_MATH1, MATH1, MATH3, NOMBRES_COMPLEXES;

package body FOURIER_GENERIQUE is

procedure ECHANTILLONNAGE(tMax: in FLOAT;
			  x: out VECTEUR_COMPLEXE; D: out FLOAT) is
-- Echantillonnage de la fonction f sur [0,tMax].
-- Si n = x'LENGTH et D = tMax/n, alors x(i) = f(i*D), pour i = 0, ..., n - 1.
PAS, t: FLOAT := 0.0;
begin
    PAS := tMax/FLOAT(x'LENGTH);
    D := PAS;
    for i in x'range
    loop
       x(i)(REEL) := f(t);
       x(i)(IMAGINAIRE) := 0.0;
       t := t + PAS;
    end loop;
end ECHANTILLONNAGE;

function SERIE_DE_FOURIER(a, b: in VECTEUR; x: in FLOAT) return FLOAT is
S: FLOAT := a(a'FIRST);
y: FLOAT := x;
begin
    for k in b'range
    loop
        S := S + a(k)*Cos(y) + b(k)*Sin(y);
        y := y + x;
    end loop;
    return S;
end SERIE_DE_FOURIER;

procedure TRACE_UNE_SERIE_DE_FOURIER(tMax: in FLOAT; a, b: in VECTEUR;
                                     NB_POINTS: in INTEGER := 300) is
m, t, x, y, Pas, Pas_x: FLOAT;
begin
    m := 2.0*PI/tMax;
    Pas := tMax/FLOAT(NB_POINTS);
    Pas_x := m*Pas;
    t := 0.0;
    x := 0.0;
    y := SERIE_DE_FOURIER(a,b,x);
    DEPLACE(t,y);
    for j in 1..NB_POINTS - 1
    loop
	t := t + Pas; -- t = j*PAS
	x := x + Pas_x; -- x = j*(2*Pi/tMax)*t
        y := SERIE_DE_FOURIER(a,b,x);
	TRACE(t,y);
    end loop;
end TRACE_UNE_SERIE_DE_FOURIER;

procedure INVERSION_BINAIRE(x: in VECTEUR_COMPLEXE;
			    y: out VECTEUR_COMPLEXE) is
-- Inversion binaire d'un vecteur complexe.
n: FLOAT := FLOAT(x'LENGTH);
p: INTEGER := INTEGER(LOG(n,2.0)); -- n = 2^p
h, Aux, c: INTEGER;
begin
    for k in x'range
    loop
        h := 0;
        Aux := x'LENGTH/2;
        for r in 0..p - 1
        loop
            c := (k/(2**r)) mod 2;
            h := h + c*Aux;
            Aux := Aux/2;
	end loop;
        y(h) := x(k);
    end loop;
end INVERSION_BINAIRE;

procedure FFT(x: in VECTEUR_COMPLEXE; y: out VECTEUR_COMPLEXE) is
-- Calcul de la transformation de Fourier discrete. Il faudra multiplier
-- par D = tMax/n pour retrouver la transformee de Fourier classique.
n: FLOAT := FLOAT(x'LENGTH);
p: INTEGER := INTEGER(LOG(n,2.0)); -- n = 2^p
m, i, j: INTEGER;
w1, w2, Aux: COMPLEXE;
xx: VECTEUR_COMPLEXE(x'range) := x;
begin
    m := x'LENGTH/2;
    for r in 0..p - 1
    loop
	w1(REEL) := 1.0; W1(IMAGINAIRE) := 0.0;
	w2(REEL) := COS(Pi/FLOAT(m)); w2(IMAGINAIRE) := -SIN(Pi/FLOAT(m));
        for L in 0..m - 1
        loop
	    for h in 0..(x'LENGTH/(2*m) - 1)
            loop
                i := L + 2*m*h;
                j := i + m;
		Aux := xx(i) - xx(j);
                Aux := Aux*w1;
		xx(i) := xx(i) + xx(j);
		xx(j) := Aux;
            end loop;
            w1 := w1*w2;
        end loop;
        m := m/2;
    end loop;
    INVERSION_BINAIRE(xx,y);
 end FFT;

procedure CALCUL_COEFFICIENTS_DE_FOURIER(y: in VECTEUR_COMPLEXE;
					 a, b: out VECTEUR) is
-- Calcul des coefficients de Fourier trigonometriques.
-- Si n = y'LENGTH, on doit avoir a'LENGTH = n/2, b'LENGTH = n/2 - 1
AUX: FLOAT;
yy: VECTEUR_COMPLEXE(y'range) := y;
begin
    Aux := 1.0/FLOAT(y'LENGTH);
    for i in y'range
    loop
       yy(i) := Aux*yy(i);
    end loop;
    a(a'FIRST) := yy(yy'FIRST)(REEL);
    for i in b'range
    loop
	a(i) := 2.0*yy(i)(REEL);
	b(i) := -2.0*yy(i)(IMAGINAIRE);
    end loop;
end CALCUL_COEFFICIENTS_DE_FOURIER;

function ENERGIE(x: in VECTEUR_COMPLEXE) return FLOAT is
-- Calcule de |z(0)|^2 + ... |z(n-1)|^2.
S: FLOAT := 0.0;
begin
    for i in x'range
    loop
	S := S + x(i)(REEL)**2 + x(i)(IMAGINAIRE)**2;
    end loop;
    return S;
end ENERGIE;

end FOURIER_GENERIQUE;
