with TEXT_IO, MATH0, COMMON_MATH1, MATH1, MATH2;
use TEXT_IO, MATH0, COMMON_MATH1, MATH1, MATH2;

package body INTEGRATION_GENERIQUE is

type TABLEAU_GAUSS is array(POSITIVE range <>) of FLOAT;

procedure TRAPEZES_DICHOTOMIQUES(a, b: in FLOAT; Iter: in INTEGER;
				 n: in out LONG_INTEGER; T: in out FLOAT) is
x, Somme, h: FLOAT;
begin
    if Iter = 1 then
       T := 0.5*(b - a)*(f(a) + f(b));
       n := 1;
    else
       h := (b - a)/FLOAT(n);
       x := a + 0.5*h;
       Somme := 0.0;
       for j in 1..n loop
	  Somme := Somme + f(x);
	  x := x + h;
       end loop;
       T := 0.5*(T + Somme*h);
       n := 2*n;
    end if;
end TRAPEZES_DICHOTOMIQUES;
-- 
function INTEGRATION_TRAPEZES(a, b: in FLOAT) return FLOAT is
MAX_ITERATION : constant NATURAL := 20;
Iter: INTEGER range 0..MAX_ITERATION := 0;
Eps: constant FLOAT := 1.0E-6;
n: LONG_INTEGER;
T, T1: FLOAT;
begin
    T1 := -1.0E+37;
    loop
      begin
	 Iter := Iter + 1;
      exception
	 when CONSTRAINT_ERROR =>
	 PUT_LINE("Stop. Methode inadaptee, convergence trop lente. ");
      end;
      TRAPEZES_DICHOTOMIQUES(a, b, Iter, n, T);
      T1 := T;
      exit when ABS(T - T1) < EPS;
    end loop;
    return T;
end INTEGRATION_TRAPEZES;
-- 
function INTEGRATION_SIMPSON(a, b: in FLOAT) return FLOAT is
MAX_ITERATION : constant NATURAL := 20;
Iter: INTEGER range 1..MAX_ITERATION := 1;
Eps: constant FLOAT := 1.0E-6;
T1, T, S, S1: FLOAT;
n: LONG_INTEGER;
begin
    TRAPEZES_DICHOTOMIQUES(a,b,Iter,n,T);
    T1 := T;
    S1 := -1.0E+37;
    loop
      begin
	 Iter := Iter + 1;
      exception
	 when CONSTRAINT_ERROR =>
	 PUT_LINE("Stop. Methode inadaptee, convergence trop lente. ");
      end;
      TRAPEZES_DICHOTOMIQUES(a,b,Iter,n,T);
      S := (4.0*T - T1)/3.0;
      T1 := T;
      S1 := S;
      exit when ABS(S - S1) < Eps;
    end loop;
    return S;
end INTEGRATION_SIMPSON;
-- 
function INTEGRATION_ROMBERG(a,b: in FLOAT) return FLOAT is
N_MAX : constant NATURAL := 15;
m: INTEGER range 0..N_MAX := 0;
Eps: constant FLOAT := 1.0E-6;
n, p: LONG_INTEGER;
R, R1, T1: FLOAT;
T: array(0..N_Max) of FLOAT;
Correct: BOOLEAN;
begin
    TRAPEZES_DICHOTOMIQUES(a,b,1,n,T1);
    T(0) := T1; -- On sort le resultat dans T1, car il est reinjecte
		-- dans la procedure TRAPEZES_DICHOTOMIQUES a l'etape
		-- suivante.
    R1 := -1.0E+37;
    -- T(0) contient l'integrale calculee avec Romberg et T(m) avec Trapezes
    loop
	begin
	   m := m + 1;
	exception
	   when CONSTRAINT_ERROR =>
	   PUT_LINE("Stop. Methode inadaptee, convergence trop lente. ");
	end;
	TRAPEZES_DICHOTOMIQUES(a,b,m + 1,n,T1);
	T(m) := T1; -- T(m) Represente T(0,m)
	p := 1;
	for j in reverse 0..m - 1 loop
	    p := 4*p;
	    T(j) := (FLOAT(p)*T(j+1) - T(j))/FLOAT(p - 1);
		    -- Represente T(m - j,j)
	end loop;
	R := T(0);
	R1 := R;
	exit when ABS(R - R1) < Eps;
     end loop;
     return R;
end INTEGRATION_ROMBERG;
-- 
function INTEGRATION_LEGENDRE(a, b: in FLOAT) return FLOAT is
S: FLOAT;
n: INTEGER := 10;
Z, C: TABLEAU_GAUSS(1..n);
begin
-- Zeros du polynome L10
   Z(1) :=  0.973906528520836;
   Z(2) :=  0.865063366695711;
   Z(3) :=  0.679409568296381;
   Z(4) :=  0.433395394128142;
   Z(5) :=  0.148874338981509;
-- Coefficients de Gauss du polynome L10
   C(1) :=  0.066671344339035;
   C(2) :=  0.149451349085659;
   C(3) :=  0.219086362542412;
   C(4) :=  0.269266719316817;
   C(5) :=  0.295524224715335;
   for k in n/2 + 1..n loop
      Z(k) := -Z(n - k + 1);
      C(k) := C(n - k + 1);
   end loop;
   for k in 1..n loop
     Z(k) := ( (b + a) + (b - a)*Z(k) )*0.5;
   end loop;
   S := 0.0;
   for k in 1..n loop
     S := S + C(k)*f(Z(k));
   end loop;
   S := (b - a)*S*0.5;
   return S;
end INTEGRATION_LEGENDRE;
-- 
function INTEGRATION_LAGUERRE return FLOAT is
n: INTEGER := 12;
A, Z: TABLEAU_GAUSS(1..n);
S: FLOAT;
begin
-- Zeros du polynome L12
   Z(1) :=    0.115722117357768;
   Z(2) :=    0.611757484515131;
   Z(3) :=    1.512610269776424;
   Z(4) :=    2.833751337743375;
   Z(5) :=    4.599227639419031;
   Z(6) :=    6.844525453111916;
   Z(7) :=    9.621316842459427;
   Z(8) :=   13.006054993306726;
   Z(9) :=   17.116855187459830;
   Z(10) :=   22.151090379405613;
   Z(11) :=   28.487967250984081;
   Z(12) :=   37.099121044468802;
-- Coefficients de Gauss du polynome L12
   A(1) :=    0.264731371069898;
   A(2) :=    0.377759275873128;
   A(3) :=    0.244082011319917;
   A(4) :=    0.090449222211577;
   A(5) :=    0.020102381154772;
   A(6) :=    0.002663973541794;
   A(7) :=    0.000203231592675;
   A(8) :=    0.000008365055856;
   A(9) :=    0.000000166849388;
   A(10) :=    0.000000001342391;
   A(11) :=    0.000000000003062;
   A(12) :=    0.000000000000001;
   S := 0.0;
   for k in 1..n loop
       S := S + A(k)*f(z(k));
   end loop;
   return S;
end INTEGRATION_LAGUERRE;
-- 
function INTEGRATION_HERMITE return FLOAT is
n: INTEGER := 10;
A, Z: TABLEAU_GAUSS(1..n);
S: FLOAT;
begin
-- Zeros du polynome H10
    Z(1) :=   -3.436159118839221;
    Z(2) :=   -2.532731674224989;
    Z(3) :=   -1.756683649298586;
    Z(4) :=   -1.036610829788826;
    Z(5) :=   -0.342901327223690;
-- Coefficients de Gauss du polynome H10 }
    A(1) :=    0.000007640432855;
    A(2) :=    0.001343645746870;
    A(3) :=    0.033874394455717;
    A(4) :=    0.240138611081685;
    A(5) :=    0.610862633734486;
    for k in n/2 + 1..n loop
	Z(k) := -Z(n - k + 1);
	A(k) := A(n - k + 1);
    end loop;
    S := 0.0;
    for k in 1..n loop
	S := S + A(k)*f(z(k));
    end loop;
    return S;
end INTEGRATION_HERMITE;
-- 
function INTEGRATION_TCHEBYCHEV return FLOAT is
n: INTEGER := 30;
t: FLOAT := PI/FLOAT(2*n);
S: FLOAT;
begin
    S := 0.0;
    for k in 1..n loop
      S := S + f(Cos(FLOAT(2*k - 1)*t));
    end loop;
    S := S*Pi/FLOAT(n);
    return S;
end INTEGRATION_TCHEBYCHEV;
-- 
end INTEGRATION_GENERIQUE;
