with MATH1, MATH3, CURVE;
use MATH1, MATH3, CURVE;

package body LAGRANGE is

function LAGRANGE_NEVILLE(x, y: VECTEUR; t: FLOAT) return FLOAT is
-- Calcul de la valeur du polynome de Lagrange en t par l'algorithme de
-- Neville.
P: VECTEUR(x'range);
begin
    for i in x'range loop
       P(i) := y(i);
       for j in reverse 0..i - 1 loop
	 P(j) := P(j+1) + (P(j+1) - P(j))*(t - x(i))/(x(i) - x(j));
       end loop;
    end loop;
    return P(0);
end LAGRANGE_NEVILLE;
-- 
procedure POLYNOME_LAGRANGE(x, y: in VECTEUR; P: out VECTEUR) is
-- Le polynome de Lagrange est ecrit sous la forme de Newton:
--  P(t) = P0 + (t - x0).P1 + ... + (t - x0)...(t - x(n-1)).Pn.
Q: VECTEUR(x'range);
begin
    for i in x'range loop
	Q(i) := y(i);
	for j in reverse 0..i - 1 loop
	  Q(j) := (Q(j+1) - Q(j))/(x(i) - x(j));
	end loop;
	P(i) := Q(0);
    end loop;
end POLYNOME_LAGRANGE;
-- 
function LAGRANGE_NEWTON(x, P: in VECTEUR; t: in FLOAT) return FLOAT is
-- Evaluation du polynome de Lagrange construit par la methode de Newton.
--  Utilisation de la methode de Horner.
Aux: FLOAT;
begin
    Aux := P(x'LAST);
    for i in reverse 0..x'LAST - 1 loop
      Aux := P(i) + Aux*(t - x(i));
    end loop;
    return Aux;
end LAGRANGE_NEWTON;
-- 
procedure COURBE_INTERPOLATION_LAGRANGE(x,y: in VECTEUR; xMin, xMax: in FLOAT;
				 C: in out COURBE) is
Pas: FLOAT;
P: VECTEUR(x'range);
begin
    POLYNOME_LAGRANGE(x,y,P);
    C.uMin := xMin;
    C.uMax := xMax;
    C.u(0) := xMin;
--    C.v(0) := LAGRANGE_NEVILLE(x,y,C.u(0));
    C.v(0) := LAGRANGE_NEWTON(x,P,C.u(0));
    C.vMin := C.v(0);
    C.vMax := C.v(0);
    Pas := (xMax - xMin)/FLOAT(C.n);
    for i in 1..C.n loop
       C.u(i) := C.u(i-1) + Pas;
--       C.v(i) := LAGRANGE_NEVILLE(x,y,C.u(i));}
       C.v(i) := LAGRANGE_NEWTON(x,P,C.u(i));
       C.vMin := MIN(C.vMin,C.v(i));
       C.vMax := MAX(C.vMax,C.v(i)) ;
    end loop;
end COURBE_INTERPOLATION_LAGRANGE;
-- 
end LAGRANGE;
