with MATH2, MATH3, COMMON_MATRIX;
use MATH2, MATH3, COMMON_MATRIX;

package body NIVEAUX is

-- On definit un maillage de [xMin,xMax]x[yMin,yMax] en posant:
-- hx = (xMax - xMin)/nx, hy = (yMax - yMin)/ny
--
--    Ŀ
--                          x  = xMin + i*hx  (0 <= i <= nx)
-- y          i
--  j                
--           y  = yMin + j*hy  (0 <= j <= ny)
--                           j
--  
--            x   x
--             i   i+1
-- On definit ensuite la matrice a nx + 1 lignes et ny + 1 colonnes M en
-- posant: M    = f(x ,y ) , pour 0 <= i <= nx, 0 <= j <= ny).
--          i,j      i  j

procedure DISCRETISE_FONCTION(f: in FONCTION;
			      xMin, xMax, yMin, yMax: in FLOAT;
			      M: in out MATRICE) is
Hx: FLOAT := (xMax - xMin)/FLOAT(M'LENGTH(1) - 1);
Hy: FLOAT := (yMax - yMin)/FLOAT(M'LENGTH(2) - 1);
x, y: FLOAT;
begin
    x := xMin;
    for i in M'range(1)
    loop
       y := yMin;
       for j in M'range(2)
       loop
	  M(i,j) := EVALUE(f,x,y);  -- M(i,j) = f(x(i),y(j))
	  y := y + hy;
       end loop;
       x := x + hx;
    end loop;
end DISCRETISE_FONCTION;
-- 
procedure TRACE_NIVEAU(XMin,XMax,YMin,YMax: in FLOAT; M: in MATRICE;
		       f: in FLOAT) is
Hx: FLOAT := (xMax - xMin)/FLOAT(M'LENGTH(1) - 1);
Hy: FLOAT := (yMax - yMin)/FLOAT(M'LENGTH(2) - 1);
x, y: FLOAT;

procedure TRACE_NIVEAU_TRIANGLE(
			X1, Y1, Z1, X2, Y2, Z2, X3, Y3, Z3, F: in FLOAT) is
U,V,W: array(1..3) of FLOAT;
TRACER: BOOLEAN;
X_ORIGINE,Y_ORIGINE,X_EXTREMITE,Y_EXTREMITE: FLOAT;
zz1: FLOAT := z1;
xx2: FLOAT := x2;
yy2: FLOAT := y2;
zz2: FLOAT := z2;
k: INTEGER;

procedure CLASSE is
j: INTEGER;
begin
  U(1) := X1; V(1) := Y1; W(1) := zz1;
  U(2) := xx2; V(2) := yy2; W(2) := zz2;
  U(3) := X3; V(3) := Y3; W(3) := Z3;
  j := 1;
  for i in 2..3
  loop
     if W(i) > zz1
     then
	j := i;
	zz1 := W(i);
     end if;
   end loop;
   zz1 :=  W(1);
   U(1) := U(j); V(1) := V(j); W(1) := W(j);
   U(j) := X1; V(j) := Y1; W(j) := zz1;
   if W(3) > W(2) then
      xx2 := U(2); yy2 := V(2); zz2 := W(2);
      U(2) := U(3); V(2) := V(3); W(2) := W(3);
      U(3) := xx2; V(3) := yy2; W(3) := zz2;
   end if;
end CLASSE;

procedure TESTE is
begin
  if ( f >= W(3) ) and ( f <= W(1) ) then
      TRACER := TRUE;
      if f > W(2) then
	 k := 1;
      else
	 k := 3;
      end if;
   end if;
end TESTE;

procedure CALCULE is
Lambda1,Lambda2:  FLOAT;
begin
  if W(1) = W(3) then
     X_ORIGINE := (U(1) + U(2) + U(3))/3.0; X_EXTREMITE := X_ORIGINE;
     Y_ORIGINE := (V(1) + V(2) + V(3))/3.0; Y_EXTREMITE := Y_ORIGINE;
     null;
  else
     if W(2) = W(k) then
	X_ORIGINE := U(2); Y_ORIGINE := V(2); X_EXTREMITE := U(k); Y_EXTREMITE := V(k);
     else
	Lambda1 := (f - W(3))/(W(1) - W(3));
	Lambda2 := (f - W(2))/(W(k) - W(2));
	X_ORIGINE := Lambda1*U(1) + (1.0 - Lambda1)*U(3);
	Y_ORIGINE := Lambda1*V(1) + (1.0 - Lambda1)*V(3);
	X_EXTREMITE := Lambda2*U(k) + (1.0 - Lambda2)*U(2);
	Y_EXTREMITE := Lambda2*V(k) + (1.0 - Lambda2)*V(2);
      end if;
  end if;
end CALCULE;

begin
  TRACER := FALSE;
  CLASSE;
  TESTE;
  if TRACER then
      CALCULE;
      TRACE_SEGMENT(X_ORIGINE,Y_ORIGINE,X_EXTREMITE,Y_EXTREMITE);
  end if;
end TRACE_NIVEAU_TRIANGLE;

begin
  y := yMin;
  for j in M'FIRST(2) + 1..M'LAST(2)
  loop
     x := xMin;
     for i in M'FIRST(1)..M'LAST(1) - 1
     loop
	TRACE_NIVEAU_TRIANGLE(x,y + hy,M(i,j),x + hx,y + hy,M(i + 1,j),
			      x,y,M(i,j - 1),f);
	 TRACE_NIVEAU_TRIANGLE(x + hx,y + hy,M(i + 1,j),
			       x + hx,y,M(i + 1,j - 1), x,y,M(i,j - 1),f);
	 x := x + hx;
     end loop;
     y := y + hy;
  end loop;
end TRACE_NIVEAU;
-- 
procedure TRACE_EQUIPOTENTIELLES(M: in MATRICE;
				 xMin, xMax, yMin, yMax: in FLOAT;
				 u0: in VECTEUR; AXES: in BOOLEAN := FALSE) is
xUnite: FLOAT := (xMax - xMin)/20.0;
yUnite: FLOAT := (yMax - yMin)/20.0;
begin
    INIT_GRAPHIQUE;
    FENETRE_GRAPHIQUE(xMin - xUnite,xMax + xUnite,
		      yMin - yUnite,yMax + yUnite);
    TITRE("Trace des equipotentielles, u(x,y) = cste");
    if AXES then
       XY_AXES(xMin,yMin,xUnite,yUnite,TRUE);
    end if;
    for i in U0'range
    loop
	TRACE_NIVEAU(xMin,xMax,yMin,yMax,M,U0(i));
    end loop;
    SORTIE_GRAPHIQUE;
end TRACE_EQUIPOTENTIELLES;

end NIVEAUX;
