with TEXT_IO, FIO, COMMON_MATH0, MATH0, MATH1, MATH2,
     MATH3, COMMON_MATRIX;
use TEXT_IO, FIO, COMMON_MATH0, MATH0, MATH1, MATH2,
     MATH3, COMMON_MATRIX;

package body EDP_PARABOLIQUES is

procedure INIT_CHALEUR(dx: in FLOAT; f: in FONCTION; U: in out MATRICE) is
x: FLOAT := 0.0;
begin
    for i in U'range(1)
    loop
       U(i,U'FIRST(2)) := EVALUE(f,x);
       x := x + dx;
    end loop;
    for j in U'range(2)
    loop
       U(U'FIRST(1),j) := 0.0;
       U(U'LAST(1),j) := 0.0;
    end loop;
end INIT_CHALEUR;
-- 
procedure EDP_CHALEUR_EXPLICITE(k, dx, dt: in FLOAT; f: in FONCTION;
				U: in out MATRICE) is
-- Calcul des temperatures par la methode explicite des differences
-- finies progressive.
r: FLOAT := k*dt/(dx**2);
begin
   INIT_CHALEUR(dx,f,U);
   for j in U'FIRST(2) + 1..U'LAST(2)
   loop
      for i in U'FIRST(1) + 1..U'LAST(1) - 1
      loop
	 U(i,j) := U(i,j-1) + r*(U(i-1,j-1) - 2.0*U(i,j-1) + U(i+1,j-1));
      end loop;
   end loop;
end EDP_CHALEUR_EXPLICITE;
-- 
procedure EDP_CHALEUR_IMPLICITE(w, k, dx, dt: in FLOAT; f: in FONCTION;
				U: in out MATRICE) is
-- Calcul des temperatures par la methode implicite des differences finies
-- avec parametre. Pour w = 1, on a la methode des differences finies
-- regressive et pour w = 1/2, c'est la methode de Crank-Nicholson.
M: MATRICE_TRIDIAGONALE(U'FIRST(1) + 1..U'LAST(1) - 1);
b: VECTEUR(U'FIRST(1)+ 1..U'LAST(1) - 1);
Aux: FLOAT;
r: FLOAT := k*dt/(dx**2);
s: FLOAT := (1.0 - w)*k*dt/(dx**2);
t: FLOAT := -w*r;
v: FLOAT := 1.0 - 2.0*t;
begin
   INIT_CHALEUR(dx,f,U);
   for i in M'range
   loop
      M(i)(1) := t;
      M(i)(3) := t;
   end loop;
   for j in U'FIRST(2) + 1..U'LAST(2)
   loop
      for i in M'range
      loop
	 M(i)(2) := v;
	 b(i) := U(i,j-1) + s*(U(i-1,j-1) - 2.0*U(i,j-1) + U(i+1,j-1));
      end loop;
      for i in M'FIRST + 1..M'LAST
      loop
	 Aux := M(i)(1)/M(i - 1)(2);
	 M(i)(2) := M(i)(2) - Aux*M(i - 1)(3);
	 b(i) := b(i) - Aux*b(i - 1);
      end loop;
      U(M'LAST,j) := b(M'LAST)/M(M'LAST)(2);
      for i in reverse M'FIRST..M'LAST - 1
      loop
	 U(i,j) := (b(i) - M(i)(3)*U(i + 1,j))/M(i)(2);
      end loop;
   end loop;
end EDP_CHALEUR_IMPLICITE;
-- 
procedure AFFICHER_TEMPERATURES(U: in MATRICE; dt: in FLOAT; p: in INTEGER) is
t: FLOAT := 0.0;
nc: INTEGER;
begin
    nc := 0;
    loop
       PUT(IMP," Temperatures a t = "); PUT(IMP,t,4,3,0); NEW_LINE(IMP);
       for i in U'range(1)
       loop
	  PUT(IMP,U(i,nc),6,6,0); PUT(IMP," ");
       end loop;
       PAUSE;
       nc := nc + p;
       t := FLOAT(nc)*dt;
       exit when (nc > U'LAST(2));
    end loop;
end AFFICHER_TEMPERATURES;
-- 
procedure COURBES_CHALEUR(U: in MATRICE; dx: in FLOAT; p: in INTEGER) is
nc: INTEGER;
x, L, yFenMin, yFenMax, xUnite, yUnite: FLOAT;
begin
   L := FLOAT(U'LENGTH(1) - 1)*dx;
   xUnite := L/20.0;
   yFenMin := 1.0E+37;
   yFenMax := -yFenMin;
   nc := U'FIRST(2);
   loop
      for i in U'range(1)
      loop
	 yFenMin := MIN(yFenMin,U(i,nc));
	 yFenMax := MAX(yFenMax,U(i,nc));
      end loop;
      nc := nc + p;
      exit when (nc > U'LAST(2));
   end loop;
   yUnite := (yFenMax - yFenMin)/20.0;
   INIT_GRAPHIQUE;
   FENETRE_GRAPHIQUE(-xUnite,L + xUnite,yFenMin - yUnite,yFenMax + yUnite);
   XY_AXES(0.0,yFenMin,xUnite,yUnite,FALSE);
   nc := U'FIRST(2);
   loop
      x := 0.0;
      DEPLACE(x,U(U'FIRST(1),nc));
      for i in U'FIRST(1) + 1..U'LAST(1)
      loop
	 x := x + dx;
	 TRACE(x,U(i,nc));
      end loop;
      nc := nc + p;
      exit when (nc > U'LAST(2));
   end loop;
   SORTIE_GRAPHIQUE;
end COURBES_CHALEUR;
-- 
end EDP_PARABOLIQUES;
