with TEXT_IO, IIO, FIO, COMMON_MATH0, MATH0, MATH1, CURVE;
use TEXT_IO, IIO, FIO, COMMON_MATH0, MATH0, MATH1, CURVE;

package body POLY is

procedure GET(P: out POLYNOME) is
begin
    for i in P'range
    loop
       ENTRER_REEL(P(i),"Coefficient Numero" & INTEGER'IMAGE(i) & " = ");
    end loop;
end GET;
-- 
procedure POLYNOME_ALEATOIRE(P: out POLYNOME) is
begin
    RANDOMIZE;
    for i in P'range
    loop
       P(i) := -20.0 + RANDOM*20.0;
    end loop;
end POLYNOME_ALEATOIRE;
-- 
PROCEDURE PUT_LINE(P: in POLYNOME; FORE: in TEXT_IO.FIELD := 6;
				   AFT: in TEXT_IO.FIELD := 4;
				   EXP: in TEXT_IO.FIELD := 0) is
begin
     FIO.PUT(IMP,P(0),FORE,AFT,EXP);
     TEXT_IO.PUT(IMP," + ");
     for j in 1..P'LAST - 1 loop
	FIO.PUT(IMP,P(j),FORE,AFT,EXP);
	TEXT_IO.PUT(IMP,"*x");
	TEXT_IO.PUT(IMP,CHARACTER'VAL(94));
	if (j < 10) then
	   IIO.PUT(IMP,j,1);
	else
	   IIO.PUT(IMP,j,2);
	end if;
	TEXT_IO.PUT(IMP," + ");
     end loop;
     FIO.PUT(IMP,P(P'LAST),FORE,AFT,EXP);
     TEXT_IO.PUT(IMP,"*x");
     TEXT_IO.PUT(IMP,CHARACTER'VAL(94));
     IIO.PUT(IMP,P'LAST);
     TEXT_IO.NEW_LINE(IMP);
end PUT_LINE;
-- 
function EVALUE(P: in POLYNOME; x: in FLOAT) return FLOAT is
-- Evaluation de P(x), par la methode de Horner
Aux: FLOAT := P(P'LAST);
BEGIN
    for i in reverse P'FIRST..P'LAST - 1 loop
       Aux := P(i) + Aux*x;
    end loop;
    return Aux;
end EVALUE;
-- 
function EVALUE_DERIVEE(P: in POLYNOME; x: in FLOAT) return FLOAT is
-- Evaluation de P'(x), par la methode de Horner
Aux: FLOAT := FLOAT(P'LAST)*P(P'LAST);
BEGIN
    for i in reverse P'FIRST + 1..P'LAST - 1 loop
       Aux := FLOAT(i)*P(i) + Aux*x;
    end loop;
    return Aux;
end EVALUE_DERIVEE;
-- 
procedure TRACE_POLYNOME(P: in POLYNOME; xMin, xMax: in FLOAT) is
Nb: constant NATURAL := 300;
Pas: FLOAT;
C: COURBE(Nb);
begin
    C.u(0) := xMin;
    C.v(0) := EVALUE(P,xMin);
    C.uMin := xMin;
    C.uMax := xMax;
    C.vMin := 1.0E+37;
    C.vMax := -C.vMin;
    Pas := (xMax - xMin)/FLOAT(Nb);
    for i in 1..C.n loop
       C.u(i) := C.u(i - 1) + Pas;
       C.v(i) := EVALUE(P,C.u(i));
       C.vMin := MIN(C.v(i),C.vMin);
       C.vMax := MAX(C.v(i),C.vMax);
    end loop;
    TRACE_UNE_COURBE(C);
end TRACE_POLYNOME;
-- 
procedure NEWTON_MAEHLY(P: in POLYNOME; x: in out VECTEUR) is
EPS: constant FLOAT := 1.0E-12;
MAX_ITER: constant INTEGER := 500;
k: INTEGER;
x0, xk, xkP1, U, V, S: FLOAT;
begin
    x0 := abs(P(P'FIRST)/P(P'LAST));
    for i in P'FIRST + 1..P'LAST
    loop
       x0 := MAX(x0, 1.0 + abs(P(i))/P(P'LAST));
    end loop;
    for m in P'FIRST + 1..P'LAST
    loop
	xkP1 := x0;
	for j in reverse 1..2
	loop
	   loop
	      xk := xkP1;
	      U := EVALUE(P,xk);
	      S := 0.0;
	      if (abs(U) > Eps) then
		 V := EVALUE_DERIVEE(P,xk);
		 for i in 1..m - 1
		 loop
		    S := S + 1.0/(xk - x(i));
		 end loop;
		 V := U/(V - U*S);
		 xkP1 := xk - FLOAT(j)*V;
	      end if;
	      exit when ( (abs(U) < Eps) or (xkP1 >= xk) ) or (k = MAX_ITER);
	   end loop;
	   if (k = MAX_ITER) then
	      raise ERREUR_CONVERGENCE;
	   end if;
	end loop;
	x(m) := xk;
    end loop;
end NEWTON_MAEHLY;

procedure BAIRSTOW(P: in POLYNOME; x: out VECTEUR_COMPLEXE) is
type VECTEUR2 is array(INTEGER range <>) of FLOAT;
n: INTEGER := P'LAST;
k: INTEGER;
A: POLYNOME(P'range);
B: VECTEUR2(-2..n);
z1, z2: COMPLEXE;
pp, q: FLOAT;

procedure CALCULE_COEFF_BK(n: in INTEGER; A: in POLYNOME; p, q: in FLOAT;
			   B: in out VECTEUR2) is
begin
    B := (others => 0.0);
    for k in reverse 0..n
    loop
       B(k - 2) := A(k) - p*B(k - 1) - q*B(k);
    end loop;
end CALCULE_COEFF_BK;

procedure CALCULE_COEFF_CK(n: in INTEGER; B: in VECTEUR2; p, q: in FLOAT;
			   C: in out VECTEUR2) is
begin
    C := (others => 0.0);
    for k in reverse 0..n
    loop
       C(k - 1) := B(k - 1) + p*C(k) + q*C(k + 1);
    end loop;
end CALCULE_COEFF_CK;

procedure DIVISION(n: in INTEGER; A: in POLYNOME; B: in out VECTEUR2;
		   p, q: in out FLOAT) is
Eps: constant FLOAT := 1.0E-6;
MAX_ITER: constant INTEGER := 500;
C: VECTEUR2(-1..n + 1);
k, Iter: INTEGER := 0;
D, U, V, dp, dq: FLOAT;
begin
    p := 0.0; q := 0.0;
    loop
       Iter := Iter + 1;
       CALCULE_COEFF_BK(n,A,p,q,B);
       CALCULE_COEFF_CK(n,B,p,q,C);
       D := C(0)**2 - C(1)*C(-1) + C(1)*B(-1);
       U :=  C(0)*B(-1) - C(1)*B(-2);
       V := C(0)*B(-2) + B(-1)**2 - B(-1)*C(-1);
       dp := U/D; dq := V/D;
       p := p + dp; q := q + dq;
       exit when ( (abs(dp) < Eps) and (abs(dq) < Eps) ) or (Iter = MAX_ITER);
    end loop;
    if (k = MAX_ITER) then
	raise ERREUR_CONVERGENCE;
    end if;
end DIVISION;

procedure ResolEquDegre2(p, q: in FLOAT; z1, z2: in out COMPLEXE) is
-- Resolution de x**2 + p*x + q = O
D: FLOAT := p**2 - 4.0*q;
begin
    if (D = 0.0) then
       z1(Reel) := -p*0.5; z1(Imaginaire) := 0.0;
       z2 := z1;
    elsif (D > 0.0) then
       z1(Reel) := (-p - SQRT(D))*0.5; z2(Imaginaire) := 0.0;
       z2(Reel) := (-p + SQRT(D))*0.5; z2(Imaginaire) := 0.0;
    else
       D := -D;
       z1(Reel) := -p*0.5;
       z1(Imaginaire) := -SQRT(D)*0.5;
       z2(Reel) := z1(Reel);
       z2(Imaginaire) := -z1(Imaginaire);
    end if;
end ResolEquDegre2;

begin
    for i in P'FIRST..P'LAST - 1
    loop
       A(i) := P(i)/P(P'LAST);
    end loop;
    A(P'LAST) := 1.0;
    k := 0;
    loop
       DIVISION(n,A,B,pp,q);
       n := n - 2;
       for i in 0..n
       loop
	  A(i) := B(i);
       end loop;
       ResolEquDegre2(pp,q,z1,z2);
       k := k + 1; x(k) := z1;
       k := k + 1; x(k) := z2;
       exit when (n <= 2);
    end loop;
    if (n = 2) then
	ResolEquDegre2(A(1),A(0),z1,z2);
	k := k + 1; x(k) := z1;
	k := k + 1; x(k) := z2;
    end if;
    if (n = 1) then
	k := k + 1; x(k)(Reel) := -A(0)/A(1);
	x(k)(Imaginaire) := 0.0;
    end if;
end BAIRSTOW;

end POLY;