with TEXT_IO, IIO, FIO, COMMON_MATH0, MATH0, MATH1, MATH2;
use TEXT_IO, IIO, FIO, COMMON_MATH0, MATH0, MATH1, MATH2;

package body EQUADIFF_1Q_GENERIQUE is

Eps0: constant FLOAT := 1.0E-6;
NB_PAS: constant INTEGER := 400;

--  RESOLUTION D'UNE EQUATION DIFFERENTIELLE D'ORDRE q

procedure RK4_1Q(X_INITIAL, h: in FLOAT;
                 Y_INITIAL: in VARIABLE_2;
		 X_FINAL: in out FLOAT; Y_FINAL: in out VARIABLE_2) is
k1, k2, k3, k4: VARIABLE_2(Y_INITIAL'range);
begin
    for j in Y_INITIAL'FIRST..Y_INITIAL'LAST - 1
    loop
       k1(j) := h*Y_INITIAL(j+1);
    end loop;
    k1(Y_INITIAL'LAST) := h*F1Q(X_INITIAL,Y_INITIAL);
    for j in Y_INITIAL'range
    loop
       Y_FINAL(j) := Y_INITIAL(j) + k1(j)*0.5;
    end loop;
    X_FINAL := X_INITIAL + h*0.5;
    for j in Y_INITIAL'FIRST..Y_INITIAL'LAST - 1
    loop
       k2(j) := h*Y_FINAL(j+1);
    end loop;
    k2(Y_INITIAL'LAST) := h*F1Q(X_FINAL,Y_FINAL);
    for j in Y_INITIAL'range
    loop
       Y_FINAL(j) := Y_INITIAL(j) + k2(j)*0.5;
    end loop;
    for j in Y_INITIAL'FIRST..Y_INITIAL'LAST - 1
    loop
       k3(j) := h*Y_FINAL(j+1);
    end loop;
    k3(Y_INITIAL'LAST) := h*F1Q(X_FINAL,Y_FINAL);
    for j in Y_INITIAL'range
    loop
       Y_FINAL(j) := Y_INITIAL(j) + k3(j);
    end loop;
    X_FINAL := X_INITIAL + h;
    for j in Y_INITIAL'FIRST..Y_INITIAL'LAST - 1
    loop
       k4(j) := h*Y_FINAL(j+1);
    end loop;
    k4(Y_INITIAL'LAST) := h*F1Q(X_FINAL,Y_FINAL);
    for j in Y_INITIAL'range
    loop
      Y_FINAL(j) := Y_INITIAL(j) + (k1(j) + 2.0*(k2(j) + k3(j)) + k4(j))/6.0;
    end loop;
end RK4_1Q;

procedure RK4_1Q_CONTROLE(X_INITIAL, h0: in FLOAT;
			  Y_INITIAL: in VARIABLE_2;
			  X_FINAL: in out FLOAT; Y_FINAL: in out VARIABLE_2;
			  H_INITIAL: in out FLOAT) is
y_Temporaire1, y_Temporaire2: VARIABLE_2(Y_INITIAL'range);
Eps: FLOAT;
begin
    loop
	RK4_1Q(X_INITIAL,H_INITIAL*0.5,Y_INITIAL,X_FINAL,y_Temporaire1);
	RK4_1Q(X_INITIAL + H_INITIAL*0.5,H_INITIAL*0.5,y_Temporaire1,
               X_FINAL,y_Temporaire2);
	RK4_1Q(X_INITIAL,H_INITIAL,Y_INITIAL,X_FINAL,y_Temporaire1);
	Eps := 0.0;
	for j in Y_INITIAL'range
	loop
	  Eps := MAX(Eps,abs(y_Temporaire2(j) - y_Temporaire1(j)));
	end loop;
	Eps := Eps/Eps0;
	if (Eps > 1.0) then
	  H_INITIAL := H_INITIAL*Eps**(-0.2);
	elsif H_INITIAL < h0*0.5 then
	  H_INITIAL := H_INITIAL*2.0;
	end if;
	if (Eps < 1.0) then
	   exit;
	end if;
    end loop;
    for j in Y_INITIAL'range
    loop
       Y_FINAL(j) := (16.0*y_Temporaire2(j) - y_Temporaire1(j))/15.0;
    end loop;
end RK4_1Q_CONTROLE;

procedure RESOLUTION_RK4_1Q(a, b: in FLOAT;
			    Y_INITIAL: in VARIABLE_2; AFFICHER: in BOOLEAN;
			    x, y: in out VECTEUR) is
h0: FLOAT;
Y0, Y1: VARIABLE_2(Y_INITIAL'range) := Y_INITIAL;
begin
  h0 := (b - a)/FLOAT(x'LENGTH - 1);
  x(x'FIRST) := a;
  y(y'FIRST) := Y_INITIAL(Y_INITIAL'FIRST);
  if AFFICHER then
     PUT(IMP,"x(0) = ");
     PUT(IMP,x(x'FIRST),6,3,0);
     PUT(IMP,"  y(0) = ");
     PUT(IMP,y(y'FIRST),6,6,0);
     NEW_LINE(IMP);
  end if;
  for i in x'FIRST + 1..x'LAST
  loop
    RK4_1Q(x(i-1),h0,Y0,x(i),Y1);
    y(i) := Y1(Y1'FIRST);
    Y0 := Y1;
    if AFFICHER then
       PUT(IMP,"x(");
       PUT(IMP,i,3);
       PUT(IMP,") = ");
       PUT(IMP,x(i),6,3,0);
       PUT(IMP,"  y(");
       PUT(IMP,i,3);
       PUT(IMP,") = ");
       PUT(IMP,y(i),6,6,0);
       NEW_LINE(IMP);
       if (i MOD 20) = 0 then
	  PAUSE;
       end if;
    end if;
  end loop;
end RESOLUTION_RK4_1Q;

procedure RESOLUTION_RK4_1Q_CONTROLE(a, b: in FLOAT;
				     Y_INITIAL: in VARIABLE_2;
				     AFFICHER: in BOOLEAN;
				     x, y: in out VECTEUR) is
Y0, Y1: VARIABLE_2(Y_INITIAL'range) := Y_INITIAL;
CONTINUER: BOOLEAN := TRUE;
h0, H_INITIAL: FLOAT;
i: INTEGER;
begin
  x(x'FIRST) := a;
  y(y'FIRST) := Y_INITIAL(Y_INITIAL'FIRST);
  if AFFICHER then
     PUT(IMP,"x(0) = ");
     PUT(IMP,x(x'FIRST),6,3,0);
     PUT(IMP,"  y(0) = ");
     PUT(IMP,y(y'FIRST),6,6,0);
     NEW_LINE(IMP);
  end if;
  h0 := (b - a)/FLOAT(NB_PAS);
  i := x'FIRST;
  H_INITIAL := h0;
  while CONTINUER
  loop
    RK4_1Q_CONTROLE(x(i),h0,Y0,x(i+1),Y1,H_INITIAL);
    y(i+1) := Y1(Y1'FIRST);
    Y0 := Y1;
    i := i + 1;
    if AFFICHER then
       PUT(IMP,"x(");
       PUT(IMP,i,3);
       PUT(IMP,") = ");
       PUT(IMP,x(i),6,3,0);
       PUT(IMP,"  y(");
       PUT(IMP,i,3);
       PUT(IMP,") = ");
       PUT(IMP,y(i),6,6,0);
       NEW_LINE(IMP);
       if (i MOD 20) = 0 then
	  PAUSE;
       end if;
    end if;
    CONTINUER := (x(i) + H_INITIAL<= b) and (i < x'LAST - 1);
  end loop;
  h0 := MIN(h0,b - x(i));
  RK4_1Q(x(i),h0,Y0,x(i+1),Y1);
  y(i+1) := Y1(Y1'FIRST);
  i := i + 1;
  if AFFICHER then
     PUT(IMP,"x(");
     PUT(IMP,i,3);
     PUT(IMP,") = ");
     PUT(IMP,x(i),6,3,0);
     PUT(IMP,"  y(");
     PUT(IMP,i,3);
     PUT(IMP,") = ");
     PUT(IMP,y(i),6,6,0);
     NEW_LINE(IMP);
     if (i MOD 20) = 0 then
	PAUSE;
     end if;
  end if;
  for j in i + 1..x'LAST loop
     x(j) := b + h0;
     y(j) := y(i);
  end loop;
end RESOLUTION_RK4_1Q_CONTROLE;

end EQUADIFF_1Q_GENERIQUE;
