with TEXT_IO, IIO, FIO, COMMON_MATH0, MATH0, MATH1, MATH2;
use TEXT_IO, IIO, FIO, COMMON_MATH0, MATH0, MATH1, MATH2;

package body EQUADIFF_P1_GENERIQUE is

Eps0: constant FLOAT := 1.0E-6;
NB_PAS: constant INTEGER := 400;

-- RESOLUTION D'UN SYSTEME DIFFERENTIELLE D'ORDRE 1

procedure RK4_P1(X_INITIAL, h: in FLOAT;
		 Y_INITIAL: in VARIABLE_2;
		 X_FINAL: in out FLOAT; Y_FINAL: in out VARIABLE_2) is
k1, k2, k3, k4: VARIABLE_2(Y_INITIAL'range);
begin
    for j in Y_INITIAL'range
    loop
      k1(j) := h*FP1(j,X_INITIAL,Y_INITIAL);
    end loop;
    X_FINAL := X_INITIAL + h*0.5;
    for j in Y_INITIAL'range
    loop
      Y_FINAL(j) := Y_INITIAL(j) + k1(j)*0.5;
    end loop;
    for j in Y_INITIAL'range
    loop
      k2(j) := h*FP1(j,X_FINAL,Y_FINAL);
    end loop;
    for j in Y_INITIAL'range
    loop
      Y_FINAL(j) := Y_INITIAL(j) + k2(j)*0.5;
    end loop;
    for j in Y_INITIAL'range
    loop
      k3(j) := h*FP1(j,X_FINAL,Y_FINAL);
    end loop;
    X_FINAL := X_INITIAL + h;
    for j in Y_INITIAL'range
    loop
      Y_FINAL(j) := Y_INITIAL(j) + k3(j);
    end loop;
    for j in Y_INITIAL'range
    loop
      k4(j) := h*FP1(j,X_FINAL,Y_FINAL);
    end loop;
    for j in Y_INITIAL'range
    loop
      Y_FINAL(j) := Y_INITIAL(j) + (k1(j) + 2.0*(k2(j) + k3(j)) + k4(j))/6.0;
    end loop;
end RK4_P1;

procedure RK4_P1_CONTROLE(X_INITIAL, h0: in FLOAT;
			  Y_INITIAL: in VARIABLE_2;
			  X_FINAL, H_INITIAL: in out FLOAT;
			  Y_FINAL: in out VARIABLE_2) is
y_Temporaire1, y_Temporaire2: VARIABLE_2(Y_INITIAL'range);
Eps: FLOAT;
begin
    loop
	RK4_P1(X_INITIAL,H_INITIAL*0.5,Y_INITIAL,X_FINAL,y_Temporaire1);
	RK4_P1(X_INITIAL + H_INITIAL*0.5,H_INITIAL*0.5,y_Temporaire1,
	       X_FINAL,y_Temporaire2);
	RK4_P1(X_INITIAL,H_INITIAL,Y_INITIAL,X_FINAL,y_Temporaire1);
	Eps := 0.0;
	for j in Y_INITIAL'range
	loop
	    Eps := MAX(Eps,abs(y_Temporaire2(j) - y_Temporaire1(j)));
	end loop;
	Eps := Eps/Eps0;
	if (Eps > 1.0) then
	  H_INITIAL := H_INITIAL*Eps**(-0.2);
	elsif H_INITIAL < h0/2.0 then
	  H_INITIAL := H_INITIAL*2.0;
	end if;
	if (Eps < 1.0) then
	   exit;
	end if;
    end loop;
    for j in Y_INITIAL'range
    loop
      Y_FINAL(j) := (16.0*y_Temporaire2(j) - y_Temporaire1(j))/15.0;
    end loop;
end RK4_P1_CONTROLE;

procedure RESOLUTION_RK4_P1(a, b: in FLOAT;
			    Y_INITIAL: in VARIABLE_2; AFFICHER: in BOOLEAN;
			    x: in out VECTEUR; y: in out MATRICE) is
h0: FLOAT;
Y0, Y1: VARIABLE_2(Y_INITIAL'range) := Y_INITIAL;
begin
  h0 := (b - a)/FLOAT(x'LENGTH - 1);
  x(x'FIRST) := a;
  for j in y'range(2)
  loop
      y(y'FIRST(1),j) := Y_INITIAL(j);
  end loop;
  if AFFICHER then
     for j in Y_INITIAL'range
     loop
       PUT(IMP,"y");
       PUT(IMP,j);
       PUT(IMP,"(");
       PUT(IMP,a,6,3,0);
       PUT(IMP,") = ");
       PUT(IMP,Y_INITIAL(j),6,6,0);
       NEW_LINE(IMP);
     end loop;
     NEW_LINE(IMP);
  end if;
  for i in x'FIRST + 1..x'LAST
  loop
      RK4_P1(x(i - 1),h0,Y0,x(i),Y1);
      for j in y'range(2)
      loop
	  y(i,j) := Y1(j);
	  Y0(j) := y1(j);
      end loop;
      if AFFICHER then
	 for j in Y_INITIAL'range
	 loop
	    PUT(IMP,"y");
	    PUT(IMP,j);
	    PUT(IMP,"(");
	    PUT(IMP,x(i),6,3,0);
	    PUT(IMP,") = ");
	    PUT(IMP,y(i,j),6,6,0);
	    NEW_LINE(IMP);
	 end loop;
	 if (i mod 5) = 0 then
	    PAUSE;
	 end if;
	 NEW_LINE(IMP);
      end if;
  end loop;
end RESOLUTION_RK4_P1;

procedure RESOLUTION_RK4_P1_CONTROLE(a, b: in FLOAT;
			    Y_INITIAL: in VARIABLE_2; AFFICHER: in BOOLEAN;
			    x: in out VECTEUR; y: in out MATRICE) is
h0, H_INITIAL: FLOAT;
CONTINUER: BOOLEAN := TRUE;
i: INTEGER;
Y0, Y1: VARIABLE_2(Y_INITIAL'range) := Y_INITIAL;
begin
  h0 := (b - a)/FLOAT(NB_PAS);
  for j in Y_INITIAL'range
  loop
     y(y'FIRST(1),j) := Y_INITIAL(j);
  end loop;
  i := x'FIRST;
  x(i) := a;
  if AFFICHER then
     for j in Y_INITIAL'range
     loop
       PUT(IMP,"y");
       PUT(IMP,j);
       PUT(IMP,"(");
       PUT(IMP,x(i),6,3,0);
       PUT(IMP,") = ");
       PUT(IMP,y(y'FIRST(1),j),6,6,0);
       NEW_LINE(IMP);
     end loop;
     NEW_LINE(IMP);
  end if;
  H_INITIAL := h0;
  while CONTINUER
  loop
      i := i + 1;
      RK4_P1_CONTROLE(x(i - 1),h0,Y0,x(i),H_INITIAL,Y1);
      for j in Y_INITIAL'range
      loop
	  y(i,j) := Y1(j);
	  Y0(j) := Y1(j);
      end loop;
      if AFFICHER then
	 for j in Y_INITIAL'range
	 loop
	    PUT(IMP,"y");
	    PUT(IMP,j);
	    PUT(IMP,"(");
	    PUT(IMP,x(i),6,3,0);
	    PUT(IMP,") = ");
	    PUT(IMP,y(i,j),6,6,0);
	    NEW_LINE(IMP);
	 end loop;
	 NEW_LINE(IMP);
	 if (i mod 5) = 0 then
	     PAUSE;
	 end if;
      end if;
      CONTINUER := (x(i) + H_INITIAL <= b) AND (i < x'LAST - 1);
   end loop;
   i := i + 1;
   h0 := Min(h0,b - x(i - 1));
   RK4_P1(x(i - 1),h0,Y0,x(i),Y1);
   for j in Y_INITIAL'range
   loop
      y(i,j) := Y1(j);
   end loop;
   if AFFICHER then
      for j in Y_INITIAL'range
      loop
	 PUT(IMP,"y");
	 PUT(IMP,j);
	 PUT(IMP,"(");
	 PUT(IMP,x(i),6,3,0);
	 PUT(IMP,") = ");
	 PUT(IMP,y(i,j),6,6,0);
	 NEW_LINE(IMP);
      end loop;
   end if;
   for j in i + 1..x'LAST loop
      x(j) := b + h0;
      for k in Y_INITIAL'range
      loop
	 y(j,k) := y(i,k);
      end loop;
   end loop;
end RESOLUTION_RK4_P1_CONTROLE;

end EQUADIFF_P1_GENERIQUE;
