with TEXT_IO, IIO, FIO, COMMON_MATH0, MATH0, MATH1, MATH2;
use TEXT_IO, IIO, FIO, COMMON_MATH0, MATH0, MATH1, MATH2;

package body EQUADIFF_PQ_GENERIQUE is

Eps0: constant FLOAT := 1.0E-6;
NB_PAS: constant INTEGER := 400;

--  RESOLUTION D'UN SYSTEME DIFFERENTIELLE D'ORDRE Q

procedure RK4_PQ(X_INITIAL, h: in FLOAT;
		 Y_INITIAL: in MATRICE;
		 X_FINAL: in out FLOAT; Y_FINAL: out MATRICE) is
p: INTEGER := Y_INITIAL'LENGTH(1);
q: INTEGER := Y_INITIAL'LENGTH(2);
YI, YF, k1, k2, k3, k4: VARIABLE_2(1..p*q);
k: INTEGER;
begin
    k := 0;
    for j in 1..p
    loop
       for i in 0..q - 1
       loop
	  k := k + 1; -- k = q*(j - 1) + i + 1
	  YI(k) := Y_INITIAL(j,i);
       end loop;
    end loop;
    k := 0;
    for j in 1..p
    loop
       for i in 0..q - 2
       loop
	  k := k + 1; -- k := q*(j - 1) + i + 1;
	  k1(k) := h*YI(k + 1);
       end loop;
       k := k + 1; -- k := q*(j - 1) + q
       k1(k) := h*FPQ(j,X_INITIAL,YI);
    end loop;
    X_FINAL := X_INITIAL + h*0.5;
    for j in 1..p*q
    loop
       YF(j) := YI(j) + k1(j)*0.5;
    end loop;
    k := 0;
    for j in 1..p
    loop
	for i in 0..q - 2
	loop
	    k := k + 1;
	    k2(k) := h*YF(k + 1);
	end loop;
	k := k + 1;
	k2(k) := h*FPQ(j,X_FINAL,YF);
    end loop;
    for j in 1..p*q
    loop
       YF(j) := YI(j) + k2(j)*0.5;
    end loop;
    k := 0;
    for j in 1..p
    loop
	for i in 0..q - 2
	loop
	    k := k + 1;
	    k3(k) := h*YF(k + 1);
	end loop;
	k := k + 1;
	k3(k) := h*FPQ(j,X_FINAL,YF);
    end loop;
    X_FINAL := X_INITIAL + h;
    for j in 1..p*q
    loop
       YF(j) := YI(j) + k3(j);
    end loop;
    k := 0;
    for j in 1..p
    loop
	for i in 0..q - 2
	loop
	    k := k + 1;
	    k4(k) := h*YF(k + 1);
	end loop;
	k := k + 1;
	k4(k) := h*FPQ(j,X_FINAL,YF);
    end loop;
    k := 0;
    for j in 1..p
    loop
	for i in 0..q - 1
	loop
	    k := k + 1;
	    Y_FINAL(j,i) := YI(k)
			 + (k1(k) + 2.0*(k2(k) + k3(k)) + k4(k))/6.0;
	end loop;
    end loop;
end RK4_PQ;

procedure RK4_PQ_CONTROLE(X_INITIAL, h0: in FLOAT;
		 Y_INITIAL: in MATRICE;
		 X_FINAL, H_INITIAL: in out FLOAT;
		 Y_FINAL: out MATRICE) is
Y_TEMPORAIRE1, Y_TEMPORAIRE2: MATRICE(Y_INITIAL'range(1),Y_INITIAL'range(2));
p: INTEGER := Y_INITIAL'LENGTH(1);
q: INTEGER := Y_INITIAL'LENGTH(2);
Eps: FLOAT;
begin
    loop
	RK4_PQ(X_INITIAL,H_INITIAL*0.5,Y_INITIAL,X_FINAL,y_Temporaire1);
	RK4_PQ(X_INITIAL + H_INITIAL*0.5,H_INITIAL*0.5,y_Temporaire1,
	       X_FINAL,y_Temporaire2);
	RK4_PQ(X_INITIAL,H_INITIAL,Y_INITIAL,X_FINAL,y_Temporaire1);
	Eps := 0.0;
	for j in 1..p
	loop
	   for i in 0..q - 1
	   loop
	     Eps := MAX(Eps,abs(y_Temporaire2(j,i) - y_Temporaire1(j,i)));
	   end loop;
	end loop;
	Eps := Eps/Eps0;
	if (Eps > 1.0) then
	   H_INITIAL := H_INITIAL*Eps**(-0.2);
	elsif H_INITIAL < h0*0.5 then
	   H_INITIAL := H_INITIAL*2.0;
	end if;
	exit when (Eps < 1.0);
    end loop;
    for j in 1..p
    loop
	for i in 0..q - 1
	loop
	  Y_FINAL(j,i) := (16.0*y_Temporaire2(j,i) - y_Temporaire1(j,i))/15.0;
	end loop;
    end loop;
end RK4_PQ_CONTROLE;

procedure RESOLUTION_RK4_PQ(a, b: in FLOAT;
			    Y_INITIAL: in MATRICE; AFFICHER: in BOOLEAN;
			    x: in out VECTEUR; y: in out MATRICE) is
p: INTEGER := Y_INITIAL'LENGTH(1);
q: INTEGER := Y_INITIAL'LENGTH(2);
h0: FLOAT;
Y0: MATRICE(1..p,0..q-1) := Y_INITIAL;
begin
  h0 := (b - a)/FLOAT(x'LENGTH - 1);
  x(x'FIRST) := a;
  for j in 1..p
  loop
      y(y'FIRST(1),j) := Y_INITIAL(j,0);
  end loop;
  if AFFICHER then
     for j in 1..p
     loop
       PUT(IMP,"y");
       PUT(IMP,j,2);
       PUT(IMP,"(");
       PUT(IMP,a,6,3,0);
       PUT(IMP,") = ");
       PUT(IMP,y(y'FIRST(1),j),6,6,0);
       NEW_LINE(IMP);
     end loop;
  end if;
  for i in x'FIRST + 1..x'LAST
  loop
      RK4_PQ(x(i - 1),h0,Y0,x(i),Y0);
      for j in 1..p
      loop
	  y(i,j) := Y0(j,0);
      end loop;
      if AFFICHER then
	 for j in 1..p
	 loop
	    PUT(IMP,"y");
	    PUT(IMP,j,2);
	    PUT(IMP,"(");
	    PUT(IMP,x(i),6,3,0);
	    PUT(IMP,") = ");
	    PUT(IMP,y(i,j),6,6,0);
	    NEW_LINE(IMP);
	 end loop;
	 NEW_LINE(IMP);
	 if (i mod 5) = 0 then
	    PAUSE;
	 end if;
      end if;
  end loop;
end RESOLUTION_RK4_PQ;

procedure RESOLUTION_RK4_PQ_CONTROLE(a, b: in FLOAT;
			    Y_INITIAL: in MATRICE; AFFICHER: in BOOLEAN;
			    x: in out VECTEUR; y: in out MATRICE) is
p: INTEGER := Y_INITIAL'LENGTH(1);
q: INTEGER := Y_INITIAL'LENGTH(2);
Y0: MATRICE(1..p,0..q - 1) := Y_INITIAL;
i: INTEGER;
h0, H_INITIAL: FLOAT;
CONTINUER: BOOLEAN := TRUE;
begin
  h0 := (b - a)/FLOAT(NB_PAS);
  x(x'FIRST) := a;
  for j in 1..p
  loop
      y(y'FIRST(1),j) := Y_INITIAL(j,0);
  end loop;
  if AFFICHER then
     for j in 1..p
     loop
       PUT(IMP,"y");
       PUT(IMP,j,2);
       PUT(IMP,"(");
       PUT(IMP,a,6,3,0);
       PUT(IMP,") = ");
       PUT(IMP,y(y'FIRST(1),j),6,6,0);
       NEW_LINE(IMP);
     end loop;
     NEW_LINE(IMP);
  end if;
  i := x'FIRST;
  H_INITIAL := h0;
  while CONTINUER
  loop
      i := i + 1;
      RK4_PQ_CONTROLE(x(i - 1),h0,Y0,x(i),H_INITIAL,Y0);
      for j in 1..p
      loop
	  y(i,j) := Y0(j,0);
      end loop;
      if AFFICHER then
	 for j in 1..p
	 loop
	    PUT(IMP,"y");
	    PUT(IMP,j,2);
	    PUT(IMP,"(");
	    PUT(IMP,x(i),6,3,0);
	    PUT(IMP,") = ");
	    PUT(IMP,y(i,j),6,6,0);
	    NEW_LINE(IMP);
	 end loop;
	 NEW_LINE(IMP);
	 if (i mod 5) = 0 then
	    PAUSE;
	 end if;
      end if;
      CONTINUER := (x(i) + H_INITIAL <= b) AND (i < x'LAST - 1);
   end loop;
   i := i + 1;
   h0 := MIN(h0,b - x(i - 1));
   RK4_PQ(x(i - 1),h0,Y0,x(i),Y0);
   for j in 1..p
   loop
       y(i,j) := Y0(j,0);
   end loop;
   if AFFICHER then
      for j in 1..p
      loop
	 PUT(IMP,"y");
	 PUT(IMP,j,2);
	 PUT(IMP,"(");
	 PUT(IMP,x(i),6,3,0);
	 PUT(IMP,") = ");
	 PUT(IMP,y(i,j),6,6,0);
	 NEW_LINE(IMP);
      end loop;
      NEW_LINE(IMP);
   end if;
   for j in i + 1..x'LAST loop
      x(j) := b + h0;
      for k in 1..p
      loop
	 y(j,k) := y(i,k);
      end loop;
   end loop;
end RESOLUTION_RK4_PQ_CONTROLE;

end EQUADIFF_PQ_GENERIQUE;
