with TEXT_IO, MATH0, MATH1, ALGEBLIN, POLY;
use MATH0, MATH1, ALGEBLIN, POLY;

package body REGRESSIONS is

procedure REGRESSION_AFFINE(x, y: in VECTEUR; a, b, Rho: in out FLOAT) is
Ex, Ey, Vx, Vy, t: FLOAT := 0.0;
begin
    a := 0.0;
    for i in x'range loop
      Ex := Ex + x(i);
      Ey := Ey + y(i);
    end loop;
    Ex := Ex/FLOAT(x'LENGTH); -- Moyenne des x(i)
    Ey := Ey/FLOAT(y'LENGTH); -- Moyenne des y(i)
    for i in x'range loop
       t := x(i) - Ex;
       Vx := Vx + t*x(i);
       a := a + t*y(i); -- n*a, ou n = x'LENGTH, est la covariance de x et y
       t := y(i) - Ey;
       Vy := Vy + t*y(i);
    end loop;
    a := a/Vx;
    Rho := a*SQRT(Vx/Vy);
    b := Ey - a*Ex;
end REGRESSION_AFFINE;
-- 
procedure SYSTEME_REGRESSION(x, y: in VECTEUR;
			     Beta: in out MATRICE; Gamma: in out VECTEUR) is
begin
    Beta := (others => (others => 0.0));
    Gamma := (others => 0.0);
    Beta(1,1) := FLOAT(x'LENGTH);
    Gamma(1) := y(1);
    for i in 2..x'LAST loop
       Gamma(1) := Gamma(1) + y(i);
    end loop;
    for i in 2..Gamma'LAST loop -- Gamma'LAST = p + 1, ou p est le degre
				-- du polynome des moindres carrees.
       Gamma(i) := y(1)*(x(1)**(i - 1));
       for j in 2..x'LAST loop
	  Gamma(i) := Gamma(i) + y(j)*(x(j)**(i - 1));
       end loop;
       for j in 1..i loop
	  Beta(i,j) := x(1)**(i + j - 2);
	  for k in 2..x'LAST loop
	      Beta(i,j) := Beta(i,j) + x(k)**(i + j - 2);
	  end loop;
       end loop;
    end loop;
    for i in 1..Gamma'LAST - 1 loop
       for j in i + 1..Gamma'LAST loop
	  Beta(i,j) := Beta(j,i);
       end loop;
    end loop;
end SYSTEME_REGRESSION;
-- 
procedure REGRESSION_POLYNOMIALE(x, y: in VECTEUR; U: out POLYNOME) is
p: NATURAL := U'LAST; -- degre de U
Gamma: VECTEUR(1..p + 1);
Beta: MATRICE(1..p + 1,1..p + 1);
begin
      SYSTEME_REGRESSION(x,y,Beta,Gamma);
      PIVOTAGE(Beta,Gamma);
      SYSTEME_TRIANGULAIRE_SUPERIEUR(Beta,Gamma,Gamma);
      for i in U'range loop
	U(i) := Gamma(i + 1);
      end loop;
end REGRESSION_POLYNOMIALE;
-- 
procedure REGRESSION_CIRCULAIRE(x, y: in VECTEUR; a, b, r: in out FLOAT) is
Ex, Ey, Vx, Vy, Cov, E, F, G: FLOAT := 0.0;
S, T, Alpha, Beta: FLOAT;
D: FLOAT range FLOAT'EPSILON..FLOAT'LAST;
begin
   for i in x'range loop
      Ex := Ex + x(i);
      Ey := Ey + y(i);
   end loop;
   Ex := Ex/FLOAT(x'LAST);
   Ey := Ey/FLOAT(y'LAST);
   for i in x'range loop
      t := x(i) - Ex;
      s := x(i)*x(i) + y(i)*y(i);
      Vx := Vx + t*x(i);
      Cov := Cov + t*y(i);
      E := E + t*s;
      t := y(i) - Ey;
      Vy := Vy + t*y(i);
      F := F + t*s;
      G := G + s;
   end loop;
   begin
      D := Vx*Vy - Cov*Cov;
   exception
      when CONSTRAINT_ERROR => -- Les points sont alignes
	 TEXT_IO.PUT_LINE("Erreur dans REGRESSION_CIRCULAIRE: Delta <= 0.0");
	 raise;
   end;
   G := G/FLOAT(x'LAST);
   Alpha := Cov*F - Vy*E;
   Beta := Cov*E - Vx*F;
   a := Alpha/D;
   b := Beta/D;
   r := G + a*Ex + b*Ey;
   r := -r;
   a := -a/2.0;
   b := -b/2.0;
   r := SQRT(a*a + b*b - r);
end REGRESSION_CIRCULAIRE;
-- 

end REGRESSIONS;

