with TEXT_IO, IIO, FIO, COMMON_MATH0, MATH0, MATH_LIB, MATRIX, ALGEBLIN;
use TEXT_IO, IIO, FIO, COMMON_MATH0, MATH0, MATH_LIB, MATRIX, ALGEBLIN;
package body SPECTRE is

MAX_ITER: constant NATURAL := 500;
EPS: constant FLOAT := 1.0E-6;
ITERATION: INTEGER;

procedure DEFLATION(A: in MATRICE; VALEURS_PROPRES: in out VECTEUR;
				   VECTEURS_PROPRES: in out MATRICE) is
-- Calcul des p premieres valeurs propres avec les vecteurs propres associes.
-- Pour j = 1, ..., p, VALEURS_PROPRES(j) est la valeur propre Nj et
-- VECTEURS_PROPRES(i,j) sont les composantes d'un vecteur propre associe.
VK, V, V_ANCIEN: VECTEUR(A'range(1));
AA: MATRICE(A'range(1),A'range(2)) := A;
AUX, AUX1: FLOAT;
NUMERO: INTEGER;
begin
    for k in VECTEURS_PROPRES'range(2)
    loop
       VECTEUR_ALEATOIRE(VK);
       ITERATION := 0;
       loop
	  ITERATION := ITERATION + 1;
	  V := AA*VK;
	  NUMERO := 1;
	  for i in 2..A'LAST(1)
	  loop
	     if abs(VK(i)) > abs(VK(NUMERO)) then
		NUMERO := i;
	     end if;
	  end loop;
	  VALEURS_PROPRES(k) := V(NUMERO)/VK(NUMERO);
	  AUX := NORME_EUCLIDIENNE(V);
	  AUX1 := 1.0/AUX;
	  V := AUX1*V;
	  V_ANCIEN := VK;
	  VK := V;
	  if ( (NORME_DU_SUP(V_ANCIEN - VK) < EPS) or (ITERATION = MAX_ITER) )
	  then exit;
	  end if;
       end loop;
       for i in A'range(1)
       loop
	  VECTEURS_PROPRES(i,k) := V(i);
       end loop;
       if (ITERATION = MAX_ITER) then
	  PUT(IMP,"Nombre d'iterations trop grand pour le calcul de ");
	  PUT(IMP,"la valeur propre Numero ");
	  PUT(IMP,k);
	  NEW_LINE(IMP);
	  PUT_LINE(IMP,"Elements propres obtenus.");
	  for j in 1..k
	  loop
	     PUT(IMP,"L");
	     PUT(IMP,j);
	     PUT(IMP," = ");
	     PUT(IMP,VALEURS_PROPRES(j),5,5,0);
	     NEW_LINE(IMP);
	     PUT_LINE(IMP,"Vecteur propre associe");
	     for i in A'range(1)
	     loop
		PUT(IMP,VECTEURS_PROPRES(i,j),8,5,0);
	     end loop;
	     NEW_LINE(IMP,2);
	     PAUSE;
	  end loop;
	  raise ERREUR_NON_CONVERGENCE;
       end if;
       for i in AA'range(1)
       loop
	  for j in AA'range(2)
	  loop
	     AA(i,j) := AA(i,j) - VALEURS_PROPRES(k)*VK(i)*VK(j);
	  end loop;
       end loop;
    end loop;
end DEFLATION;

procedure RUTISHAUSER(A: in MATRICE; VALEURS_PROPRES: in out VECTEUR) is
C, D: FLOAT;
AA, L, R: MATRICE(A'range(1),A'range(2));
begin
    AA := A;
    ITERATION := 0;
    loop
       DECOMPOSITION_LR(AA,L,R);
       for i in A'range(1)
       loop
	  VALEURS_PROPRES(i) := AA(i,i);
       end loop;
       AA := R*L;
       C := 0.0; D := 0.0;
       for i in A'range(1)
       loop
	  C := C + abs(AA(i,i) - VALEURS_PROPRES(i));
	  for j in 1..i - 1
	  loop
	     D := D + abs(AA(i,j));
	  end loop;
       end loop;
       ITERATION := ITERATION + 1;
       if ( (C < EPS) or (D < EPS) or (ITERATION = MAX_ITER) ) then
	  exit;
       end if;
    end loop;
    if (ITERATION = MAX_ITER) then
       PUT_LINE("Erreur dans RUTISHAUSER: Trop d'iterations");
       raise ERREUR_NON_CONVERGENCE;
    end if;
end RUTISHAUSER;

procedure JACOBI(A: in MATRICE; D: in out VECTEUR; V: in out MATRICE) is
Max_Iteration: constant NATURAL := 50;
NBRE_ROTATIONS, p, q: INTEGER;
Aux, Sk, Seuil, b, c, s, t, Tau: FLOAT;
CorrectJacobi: BOOLEAN;
x, y: Vecteur(A'range(1));
AA: MATRICE(A'range(1),A'range(2)) := A;
begin
    for i in A'range(1)
    loop
	x(i) := AA(i,i) ;
	D(i) := x(i);
	y(i) := 0.0;
	for j in A'range(2)
	loop
	   V(i,j) := 0.0;
	end loop;
	V(i,i) := 1.0;
    end loop;
    NBRE_ROTATIONS := 0; CorrectJacobi := FALSE; Iteration := 0;
    loop
	Iteration := ITERATION + 1;
	Sk := 0.0;
	for p in 1..A'LAST(1) - 1
	loop
	   for q in p + 1..A'LAST(1)
	   loop
	      Sk := Sk + ABS(AA(p,q));
	   end loop;
	end loop;
	CorrectJacobi := ( (Sk < EPS) or (Iteration = Max_Iteration) );
	if (Iteration < 4) then
	   Seuil := 0.2*Sk/FLOAT(A'LAST(1)*A'LAST(1));
	else
	   Seuil := EPS*EPS;
	end if;
	for p in 1..A'LAST(1) - 1
	loop
	   for q in p + 1..A'LAST(1)
	   loop
	      if ( (Iteration >= 4) and (1.0E8*ABS(AA(p,q)) < ABS(D(p)))
		  and (1.0E8*ABS(AA(p,q)) < ABS(D(q))) ) then
		 AA(p,q) := 0.0;
	      else
		 if ABS(AA(p,q)) > Seuil then
		     b := 0.5*(D(p) - D(q))/AA(p,q);
		     t := 1.0/(ABS(b) + SQRT(1.0 + b*b));
		     if (b < 0.0) then
			t := -t;
		     end if;
		     c := 1.0/SQRT(1.0 + t*t);
		     s := t*c;
		     Tau := s/(1.0 + c);
		     Aux := t*AA(p,q);
		     D(p) := D(p) + Aux;
		     D(q) := D(q) - Aux;
		     AA(p,q) := 0.0;
		     for j in 1..p - 1
		     loop
			Aux := AA(j,p);
			AA(j,p) := AA(j,p) + s*(AA(j,q) - AA(j,p)*Tau);
			AA(j,q) := AA(j,q) - S*(Aux + AA(j,q)*Tau);
		     end loop;
		     for j in p + 1..q - 1
		     loop
			Aux := AA(p,j);
			AA(p,j) := AA(p,j) + s*(AA(j,q) - AA(p,j)*Tau);
			AA(j,q) := AA(j,q) - S*(Aux + AA(j,q)*Tau);
		     end loop;
		     for j in q + 1..A'LAST(1)
		     loop
			Aux := AA(p,j);
			AA(p,j) := AA(p,j) + s*(AA(q,j) - AA(p,j)*Tau);
			AA(q,j) := AA(q,j) - S*(Aux + AA(q,j)*Tau);
		     end loop;
		     for j in A'range(1)
		     loop
			Aux := V(j,p);
			V(j,p) := V(j,p) + s*(V(j,q) - V(j,p)*Tau);
			V(j,q) := V(j,q) - s*(Aux + V(j,q)*Tau);
		     end loop;
		     NBRE_ROTATIONS := NBRE_ROTATIONS + 1;
		 end if;
	     end if;
	   end loop;
	end loop;
	PUT("Iteration Numero ");
	PUT(ITERATION);
	NEW_LINE;
	if CorrectJacobi then
	   exit;
	end if;
    end loop;
    if (Iteration = Max_Iteration) then
       NEW_LINE;
       PUT_LINE("Erreur dans Jacobi: Nombre d'iterations trop grand");
       raise ERREUR_NON_CONVERGENCE;
    end if;
end JACOBI;

procedure SOURIAU(A: in MATRICE; T: in out VECTEUR;
		  INVERSE: in out MATRICE) is
-- Le polynome caracteristique de A est
--        P(x) = t(0).x**n + t(1).x**(n-1) + ... + t(n)
-- avec t(0) = 1.0
B: MATRICE(A'range(1),A'range(2)) := (others => (others => 0.0));
begin
    t(T'FIRST) := 1.0;
    for i in A'range(1)
    loop
       B(i,i) := 1.0;
    end loop;
    for i in A'range(1)
    loop
       B := B*A;
       t(i) := -TRACE(B)/FLOAT(i);
       for j in A'range(1)
       loop
	  B(j,j) := B(j,j) + t(i);
       end loop;
       if (i = A'LAST(1) - 1) then
	  INVERSE := B;
       end if;
     end loop;
     for i in A'range(1)
     loop
	for j in A'range(2) loop
	   INVERSE(i,j) := -INVERSE(i,j)/t(A'LAST(1));
	end loop;
     end loop;
end SOURIAU;

end SPECTRE;
