with TEXT_IO, MATH0, MATH1, MATH3, CURVE, ALGEBLIN;
use TEXT_IO, MATH0, MATH1, MATH3, CURVE, ALGEBLIN;

package body SPLINE is

procedure SYSTEME_SPLINE_NATUREL(x, y: in VECTEUR;
				 T: in out MATRICE_TRIDIAGONALE;
				 v: in out VECTEUR) is
begin
    for i in v'range loop  -- Si x, y sont de dimension n,
			   -- v est de dimension n - 2 et T (n- 2)x(n - 2)
       T(i)(1) := x(i + 1) - x(i);
       T(i)(3) := x(i + 2) - x(i + 1);
       T(i)(2) := 2.0*(T(i)(1) + T(i)(3));
    v(i) := 6.0*((y(i + 2) - y(i + 1))/T(i)(3) - (y(i + 1) - y(i))/T(i)(1));
    end loop;
end SYSTEME_SPLINE_NATUREL;
-- 
procedure SYSTEME_SPLINE_PERIODIQUE(PERIODE: in FLOAT; x, y: in VECTEUR;
				    T: in out MATRICE_TRIDIAGONALE;
				    v: in out VECTEUR) is
xx, yy: VECTEUR(1..x'LAST + 1);
begin
    for i in x'range loop
       xx(i) := x(i);
       yy(i) := y(i);
    end loop;
    xx(x'LAST + 1) := x(2) + PERIODE;
    yy(y'LAST + 1) := y(2);
    SYSTEME_SPLINE_NATUREL(xx,yy,T,v);
end SYSTEME_SPLINE_PERIODIQUE;
-- 
procedure RESOLUTION_SYSTEME_SPLINE_PERIODIQUE(T: in out MATRICE_TRIDIAGONALE;
					       v: in out VECTEUR;
					       u: in out VECTEUR) is
AUX: FLOAT;
p: NATURAL := v'LENGTH;
Alpha, Beta, Gamma: VECTEUR(1..p) := (others => 0.0);
begin
    Gamma(1) := T(1)(1);
    Gamma(p - 1) := T(p - 1)(3);
    Gamma(p) := T(p)(2);
    for i in 2..p - 1 loop
       Aux := T(i)(1)/T(i - 1)(2);
       T(i)(2) := T(i)(2) - Aux*T(i - 1)(3);
       v(i) := v(i) - Aux*v(i - 1);
       Gamma(i) := Gamma(i) - Aux*Gamma(i - 1);
    end loop;
    for i in reverse 1..p - 1 loop
       Aux := T(i)(3)/T(i)(2);
       Alpha(i) := -Gamma(i)/T(i)(2) - Aux*Alpha(i + 1);
       Beta(i) := v(i)/T(i)(2) - Aux*Beta(i + 1);
    end loop;
    u(p) := (v(p) - T(p)(3)*Beta(1) - T(p)(1)*Beta(p - 1))/
	    (T(p)(3)*Alpha(1) + T(p)(1)*Alpha(p - 1) + T(p)(2));
    for i in 1..p - 1 loop
       u(i) := Alpha(i)*u(p) + Beta(i);
    end loop;
end RESOLUTION_SYSTEME_SPLINE_PERIODIQUE;
-- 
function P3(a, b, c, d, t: in FLOAT) return FLOAT is
begin
    return (a*t*t*t + b*t*t + c*t + d);
end P3;
-- 
PROCEDURE CALCUL_COEFFICIENTS(x, y, s: in VECTEUR;
			       a, b, c, d: out VECTEUR) is
h: FLOAT;
begin
    for i in a'range loop -- a'LAST = x'LAST - 1
       h := x(i + 1) - x(i);
       a(i) := (s(i + 1) - s(i))/(6.0*h);
       b(i) := 0.5*s(i);
       c(i) := (y(i + 1) - y(i))/h - h*(2.0*s(i) + s(i + 1))/6.0;
       d(i) := y(i);
    end loop;
end CALCUL_COEFFICIENTS;
-- 
PROCEDURE CALCUL_COEFFICIENTS_SPLINE_NATUREL(x, y: in VECTEUR;
					     a, b, c, d: out VECTEUR) is
p: NATURAL := x'LENGTH - 2;
s: VECTEUR(x'range);
v: VECTEUR(1..p);
T: MATRICE_TRIDIAGONALE(1..p);
begin
    SYSTEME_SPLINE_NATUREL(x,y,T,v);
    SYSTEME_TRIDIAGONAL(T,v);
    for i in 2..x'LAST - 1 loop
       s(i) := v(i - 1);
    end loop;
    if LIRE_REPONSE("Connaissez vous les derivess secondes aux extremites? ")
    then
       ENTRER_REEL(s(1),"Derivee seconde a l'origine: ");
       ENTRER_REEL(s(x'LAST),"Derivee seconde a l'extremite: ");
    else
       s(1) := 0.0;
       s(x'LAST) := 0.0;
    end if;
    CALCUL_COEFFICIENTS(x, y, s, a, b, c, d);
end CALCUL_COEFFICIENTS_SPLINE_NATUREL;
-- 
function SPL(x, y, a, b, c, d: in VECTEUR; t: in FLOAT) return FLOAT is
begin
    for i in 1..x'LAST - 1 loop
       if (x(i) < t) and then (t <= x(i + 1)) then
	  return P3(a(i),b(i),c(i),d(i),t - x(i));
       end if;
    end loop;
end SPL;
-- 
procedure COURBE_SPLINE(x, y, a, b, c, d: in VECTEUR;
			CC: in out COURBE) is
Pas: FLOAT;
begin
    CC.uMin := x(1);
    CC.uMax := x(x'LAST);
    CC.u(0) := x(1);
    CC.v(0) := y(1);
    CC.vMin := y(1);
    CC.vMax := y(1);
    Pas := (CC.uMax - CC.uMin)/FLOAT(CC.n);
    for k in 1..CC.n - 1 loop
       CC.u(k) := CC.u(k - 1) + Pas;
       CC.v(k) := SPL(x,y,a,b,c,d,CC.u(k));
       CC.vMin := MIN(CC.vMin,CC.v(k));
       CC.vMax := MAX(CC.vMax,CC.v(k));
    end loop;
    CC.u(CC.n) := x(x'LAST);
    CC.v(CC.n) := y(y'LAST);
end COURBE_SPLINE;
-- 
procedure TRACE_SPLINE_NATUREL(x, y: in VECTEUR) is
CC: COURBE(200);
a, b, c, d: VECTEUR(1..x'LAST - 1);
begin
    CALCUL_COEFFICIENTS_SPLINE_NATUREL(x,y,a,b,c,d);
    COURBE_SPLINE(x,y,a,b,c,d,CC);
    TRACE_UNE_COURBE(CC);
    for i in x'range loop
       POINT(x(i),y(i));
    end loop;
    SORTIE_GRAPHIQUE;
end TRACE_SPLINE_NATUREL;
-- 
procedure CALCUL_COEFFICIENTS_SPLINE_PERIODIQUE(PERIODE: in FLOAT;
						x, y: in VECTEUR;
						a, b, c, d: out VECTEUR) is
p: NATURAL := x'LENGTH - 1;
s: VECTEUR(x'range);
v: VECTEUR(1..p);
T: MATRICE_TRIDIAGONALE(1..p);
begin
    SYSTEME_SPLINE_PERIODIQUE(PERIODE,x,y,T,v);
    RESOLUTION_SYSTEME_SPLINE_PERIODIQUE(T,v,v);
    for i in 2..x'LAST loop
       s(i) := v(i - 1);
    end loop;
    s(1) := s(x'LAST);
    CALCUL_COEFFICIENTS(x, y, s, a, b, c, d);
end CALCUL_COEFFICIENTS_SPLINE_PERIODIQUE;
-- 
procedure TRACE_SPLINE_PERIODIQUE(PERIODE: in FLOAT; x, y: in VECTEUR) is
CC: COURBE(200);
a, b, c, d: VECTEUR(1..x'LAST - 1);
begin
    CALCUL_COEFFICIENTS_SPLINE_PERIODIQUE(PERIODE,x,y,a,b,c,d);
    COURBE_SPLINE(x,y,a,b,c,d,CC);
    TRACE_UNE_COURBE(CC);
    for i in x'range loop
       POINT(x(i),y(i));
    end loop;
    SORTIE_GRAPHIQUE;
end TRACE_SPLINE_PERIODIQUE;
-- 
procedure COURBE_SPLINE_PARAMETRIQUE(x, y: in VECTEUR; CC: in out COURBE) is
PERIODE, t, Pas: FLOAT;
Theta: VECTEUR(x'range);
ax, bx, cx, dx, ay, by, cy, dy: VECTEUR(1..x'LAST - 1);
begin
    for i in Theta'range loop
       Theta(i) := FLOAT(i);
    end loop;
    PERIODE := FLOAT(x'LENGTH - 1);
    CALCUL_COEFFICIENTS_SPLINE_PERIODIQUE(PERIODE,THETA,x,ax,bx,cx,dx);
    CALCUL_COEFFICIENTS_SPLINE_PERIODIQUE(PERIODE,THETA,y,ay,by,cy,dy);
    CC.uMin := x(1);
    CC.uMax := x(1);
    CC.u(0) := x(1);
    CC.v(0) := y(1);
    CC.vMin := CC.v(0);
    CC.vMax := CC.v(0);
    Pas := PERIODE/FLOAT(CC.n);
    t := 1.0;
    for k in 1..CC.n - 1 loop
       t := t + Pas;
       CC.u(k) := SPL(THETA,x,ax,bx,cx,dx,t);
       CC.v(k) := SPL(THETA,y,ay,by,cy,dy,t);
       CC.uMin := MIN(CC.uMin,CC.u(k));
       CC.uMax := MAX(CC.uMax,CC.u(k));
       CC.vMin := MIN(CC.vMin,CC.v(k));
       CC.vMax := MAX(CC.vMax,CC.v(k));
    end loop;
    CC.u(CC.n) := x(x'LAST);
    CC.v(CC.n) := y(y'LAST);
end COURBE_SPLINE_PARAMETRIQUE;
-- 
procedure TRACE_SPLINE_PARAMETRIQUE(x, y: in VECTEUR) is
CC: COURBE(200);
begin
    COURBE_SPLINE_PARAMETRIQUE(x,y,CC);
    TRACE_UNE_COURBE(CC);
    for i in x'range loop
       POINT(x(i),y(i));
    end loop;
    SORTIE_GRAPHIQUE;
end TRACE_SPLINE_PARAMETRIQUE;
-- 

end SPLINE;
